#if !defined(SISCANNER_H)
#define SISCANNER_H

#include "Scanner.h"		

#include <string>
using namespace std;

typedef SIResult	(SICALLTYPE* SI_GetScannerStatusProc)(SIScannerStatus *pScannerStatus);
typedef SIResult	(SICALLTYPE* SI_GetPaperStatusProc)(SIPaperStatus *pPaperStatus);
typedef SIResult	(SICALLTYPE* SI_GetButtonStatusProc)(uint32 buttonNumber, SIButtonStatus *pButtonStatus);
typedef SIResult	(SICALLTYPE* SI_GetPropertyProc)(SIProperty* pProperty);
typedef SIResult	(SICALLTYPE* SI_SetPropertyProc)(SIProperty* pProperty);
typedef SIResult	(SICALLTYPE* SI_ResetProc)();
typedef SIResult	(SICALLTYPE* SI_DiagnosticProc)(SIDiagTest test, void* pDiagInfo);
typedef SIResult	(SICALLTYPE* SI_GetLastErrorProc)();
typedef SIResult	(SICALLTYPE* SI_GetLastErrorTextProc)(char **ppErrorText);
typedef SIResult	(SICALLTYPE* SI_StartScanProc)();
typedef SIResult	(SICALLTYPE* SI_ReadImageDataProc)(uint8 *buffer, uint32 numberOfLinesToRead, uint32 pageNumber, uint32* numberOfLinesReturned);
typedef SIResult	(SICALLTYPE* SI_EndScanProc)();
typedef SIResult	(SICALLTYPE* SI_OpenInterfaceProc)(const char* pName);
typedef SIResult	(SICALLTYPE* SI_CloseInterfaceProc)();
typedef SIResult	(SICALLTYPE* SI_GetEventProc)(SIEventInfo* pEventInfo);
typedef SIResult	(SICALLTYPE* SI_CalibrateProc)(uint32 target, ProgressCallbackPtr pProgressCallback);
typedef SIResult	(SICALLTYPE* SI_IsCalibratedProc)();
typedef SIResult	(SICALLTYPE* SI_FeedPaperOutProc)();
typedef SIResult	(SICALLTYPE* SI_FeedProc)();


class SIScanner : public Scanner
{

public:
	SIScanner();

	virtual bool Attach(string DllPathname);
	virtual bool IsAttached()	
	{	return (m_hScannerDLL != NULL);	}
	virtual void Detach(void);


	SI_GetScannerStatusProc		SI_GetScannerStatusDLL;
	SI_GetPaperStatusProc		SI_GetPaperStatusDLL;
	SI_GetButtonStatusProc		SI_GetButtonStatusDLL;
	SI_GetPropertyProc			SI_GetPropertyDLL;
	SI_SetPropertyProc			SI_SetPropertyDLL;
	SI_ResetProc				SI_ResetDLL;
	SI_DiagnosticProc			SI_DiagnosticDLL;
	SI_GetLastErrorProc			SI_GetLastErrorDLL;
	SI_GetLastErrorTextProc		SI_GetLastErrorTextDLL;
	SI_StartScanProc			SI_StartScanDLL;
	SI_ReadImageDataProc		SI_ReadImageDataDLL;
	SI_EndScanProc				SI_EndScanDLL;
	SI_OpenInterfaceProc		SI_OpenInterfaceDLL;
	SI_CloseInterfaceProc		SI_CloseInterfaceDLL;
	SI_GetEventProc				SI_GetEventDLL;
	SI_CalibrateProc			SI_CalibrateDLL;
	SI_IsCalibratedProc			SI_IsCalibratedDLL;
	SI_FeedPaperOutProc			SI_FeedPaperOutDLL;
	SI_FeedProc					SI_FeedDLL;

	virtual SIResult SI_GetScannerStatus(SIScannerStatus *pScannerStatus);
	virtual SIResult SI_GetPaperStatus(SIPaperStatus *pPaperStatus);
	virtual SIResult SI_GetButtonStatus(uint32 buttonNumber, SIButtonStatus *pButtonStatus);
	virtual SIResult SI_GetProperty(SIProperty* pProperty);
	virtual SIResult SI_SetProperty(SIProperty* pProperty);
	virtual SIResult SI_Reset();
	virtual SIResult SI_Diagnostic(SIDiagTest test, void* pDiagInfo);
	virtual SIResult SI_GetLastError();
	virtual SIResult SI_GetLastErrorText(char **ppErrorText);
	virtual SIResult SI_StartScan();
	virtual SIResult SI_ReadImageData(uint8 *buffer, uint32 numberOfLinesToRead, uint32 pageNumber, uint32* numberOfLinesReturned);
	virtual SIResult SI_EndScan();
	virtual SIResult SI_OpenInterface(const char* pName);
	virtual SIResult SI_CloseInterface();
	virtual SIResult SI_GetEvent(SIEventInfo* pEventInfo);
	virtual SIResult SI_Calibrate(uint32 target, ProgressCallbackPtr pProgressCallback);
	virtual SIResult SI_IsCalibrated();
	virtual SIResult SI_FeedPaperOut();
	virtual SIResult SI_Feed();


private:
	HMODULE	m_hScannerDLL;
};

#endif

