// ScanDialog.cpp : implementation file
//

#include "stdafx.h"
#include "MiniScan.h"
#include "ScanDialog.h"
#include "Scanner.h"
#include "MainFrm.h"

#include <cassert>
#include <sstream>


typedef struct
{
	char name[40];				// The name used to open the interface with.
	char DLLFilename[20];		
} ScannerInfo;

//////////////  Define all scanners here. //////////////////////////////
//
//
////////////////////////////////////////////////////////////////////////
ScannerInfo g_scannerList[] = 
{	
	{"DocketPORT487",			"DPORT487.DLL"},
	{"DocketPORT488",			"DPORT488.DLL"},
	{"DocketPORT467",			"DPORT467.DLL"},
	{"DocketPORT468",			"DPORT468.DLL"},
	{"DocketPORT667",			"DPORT667.DLL"},
	{"DocketPORT687",			"DPORT687.DLL"},
};



// ScanDialog dialog

IMPLEMENT_DYNAMIC(ScanDialog, DialogBarEx)
ScanDialog::ScanDialog(CWnd* pParent /*=NULL*/)
	:m_lineBuffer(NULL),
	m_scanState(SS_IDLE),
	m_duplexCapable(false),
	m_eopdCapable(false),
	m_scannerIsOpen(false),
	m_prevPaperStatus(SI_PS_PAPER_IN),		// if paper is not in, this will harmlessly changed to be SI_PS_PAPER_OUT.
	m_prefeedEnabled(false)
{
	m_filename[0] = "front.bmp";
	m_filename[1] = "back.bmp";
}

ScanDialog::~ScanDialog()
{
	if(m_scannerIsOpen)
	{
		CloseScanner();
	}
	m_scanner.Detach();
}

void ScanDialog::DoDataExchange(CDataExchange* pDX)
{
	DialogBarEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SCANNER_COMBO, m_scannerNameCombo);
	DDX_Control(pDX, IDC_MODE_COMBO, m_modeCombo);
	DDX_Control(pDX, IDC_RESOLUTION_COMBO, m_resolutionCombo);
	DDX_Control(pDX, IDC_DUPLEX_CHECKBOX, m_duplexCheckbox);
	DDX_Control(pDX, IDC_EOP_CHECKBOX, m_eopdCheckbox);
	DDX_Control(pDX, IDC_SCAN_BUTTON, m_scanButton);
	DDX_Control(pDX, IDC_CALIBRATE_BUTTON, m_calibrateButton);

}


BEGIN_MESSAGE_MAP(ScanDialog, DialogBarEx)
	ON_CBN_SELCHANGE(IDC_SCANNER_COMBO, OnScannerComboSelectChanged)
	ON_BN_CLICKED(IDC_SCAN_BUTTON, OnBnClickedScanButton)
	ON_BN_CLICKED(IDC_CALIBRATE_BUTTON, OnBnClickedCalibrateButton)
	ON_WM_TIMER()
END_MESSAGE_MAP()


// ScanDialog message handlers

BOOL ScanDialog::OnInitDialogBar()
{
	DialogBarEx::OnInitDialogBar();

	int numScanners = sizeof(g_scannerList) / sizeof(ScannerInfo);
	
	// Add scanner names
	for(int i = 0; i < numScanners; i++)
	{
		m_scannerNameCombo.InsertString(i, g_scannerList[i].name);
	}
	m_scannerNameCombo.SetWindowText("Choose Scanner");


	// Other controls are grayed out until the user chooses a scanner.
	m_modeCombo.EnableWindow(FALSE);
	m_resolutionCombo.EnableWindow(FALSE);
	m_scanButton.EnableWindow(FALSE);
	m_calibrateButton.EnableWindow(FALSE);
	m_duplexCheckbox.EnableWindow(FALSE);
	m_eopdCheckbox.EnableWindow(FALSE);

	// Allocate a line buffer for reading in image data.
	m_lineBuffer = new unsigned char[BUFFER_SIZE_IN_BYTES];
	assert(m_lineBuffer);

	SetTimer(1, 1000, 0);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}
void ScanDialog::OnScannerComboSelectChanged()
{

	// If the scanner is currently open, close it.
	if(m_scannerIsOpen)
	{
		CloseScanner();
		m_scanner.Detach();
	}

	// Find out which scanner was choosen.
	// Attempt to open the scanner.
	// Populate the scan mode and resolution combo boxes with the possible choices.
	int currentScannerIndex = m_scannerNameCombo.GetCurSel();
	int numScanners = sizeof(g_scannerList) / sizeof(ScannerInfo);

	if(currentScannerIndex >= numScanners)
	{
		return;
	}

	string str;

	// Load the DLL. Use the DLL filename from the scanner list for the selected scanner.
	if(!m_scanner.Attach(g_scannerList[currentScannerIndex].DLLFilename))
	{
		str = "Could not find the file: ";
		str += g_scannerList[currentScannerIndex].DLLFilename;
		MessageBox( str.c_str());
		SetStatusText(str);
	}

	// Attempt to open the scanner if it's connected. It's OK if it's not.
	if(OpenScanner())
	{

		// Retrieve the property values from the scanner and populate the controls.
		GetScannerProperties();
	}

}

bool ScanDialog::OpenScanner()
{

	// Make sure the scanner DLL is loadeded.
	if(!m_scanner.IsAttached())
	{
		return false;
	}

	string str;
	SIResult result;
	
	int currentScannerIndex = m_scannerNameCombo.GetCurSel();

	result = m_scanner.SI_OpenInterface(g_scannerList[currentScannerIndex].name);

	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		m_scanner.SI_GetLastErrorText(&pErrorText);
		
		str = "Could not open scanner interface: \n";
		str += pErrorText;
		//MessageBox(str.c_str());
		SetStatusText("Scanner is not connected.");

		return false;
	}

	GetScannerProperties();

	m_duplexCheckbox.SetCheck(m_duplexCapable ? TRUE : FALSE);	// duplex turned on by default
	m_eopdCheckbox.SetCheck(m_eopdCapable ? TRUE : FALSE);		// eop detect turned on by default


	EnableUIControls();

	m_scannerIsOpen = true;
	SetStatusText("Scanner is connected.");
	return true;
}

void ScanDialog::EnableUIControls(BOOL enable)
{

	m_scannerNameCombo.EnableWindow(enable);
	m_modeCombo.EnableWindow(enable);
	m_resolutionCombo.EnableWindow(enable);
	m_scanButton.EnableWindow(enable);
	m_calibrateButton.EnableWindow(enable);
	m_duplexCheckbox.EnableWindow(m_duplexCapable ? enable : FALSE);
	m_eopdCheckbox.EnableWindow(m_eopdCapable ? enable : FALSE);

}


void ScanDialog::CloseScanner()
{
	if(m_scannerIsOpen)
	{
		m_scanner.SI_CloseInterface();
		m_scannerIsOpen = false;
	}

	// Clear mode and resolution combo boxes and gray out the controls.
	m_modeCombo.ResetContent();			// clear all items from the listbox
	m_resolutionCombo.ResetContent();

	EnableUIControls(FALSE);

	// Leave scanner name combo enabled
	m_scannerNameCombo.EnableWindow(TRUE);

}

/** \brief Retrieves all the possible scanner modes and resolution and 
			populates the comboboxes with the choices.

	\retval true If the modes and resolution we retrieved successfully.
	\retval false If the scanner DLL could not be opened or the interface could not be opened.
*/
bool ScanDialog::GetScannerProperties()
{

	string str;	// for misc. use
	SIResult result;

	// Retrieve scan modes. This scanner property always exists.
	SIProperty prop;
	int currentSelection = 0;

	prop.propertyID = SIP_SCAN_MODE;
	result = m_scanner.SI_GetProperty(&prop);
	assert((result == SIR_SUCCESS) && (prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));


	m_modeCombo.ResetContent();

	for(int i = 0; i < prop.list.numItems; i++)
	{
		// convert scan mode ID to string
		switch(prop.list.items.piVal[i])
		{
			case SI_SCANMODE_BW:	str = "B&W";	break;
			case SI_SCANMODE_GRAY:	str = "Gray";	break;
			case SI_SCANMODE_RGB:	str = "Color";	break;
			default:				
				str = "Unsupported mode"; 
				assert(false); 
				break;
		};


		// Add human-readable scan mode string to the combo box.
		m_modeCombo.InsertString(i, str.c_str());

		// Insert the scan mode identifier as combo box data so we don't have to 
		// translate back from string to scan mode ID when the user selects one.
		m_modeCombo.SetItemData(i, prop.list.items.piVal[i]);	

		// Remember which index is the current selection.
		if(prop.list.items.piVal[i] == prop.list.current.iVal)
		{
			currentSelection = i;
		}
	}

	m_modeCombo.SetCurSel(currentSelection);


	// Retive the scanner resolutions. The resolution property always exists.
	// Assume X and Y resolutions are the same.
	prop.propertyID = SIP_XRESOLUTION;
	result = m_scanner.SI_GetProperty(&prop);
	assert((result == SIR_SUCCESS) && (prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	stringstream ss;
	ss << dec;		// decimal representation

	m_resolutionCombo.ResetContent();

	for(int j = 0; j < prop.list.numItems; j++)
	{
		// convert numerical resolution to string
		ss.str("");		// clear
		ss << prop.list.items.piVal[j];

		// Add human-readable resolution string to the combo box.
		m_resolutionCombo.InsertString(j, ss.str().c_str());

		// Insert the numberical resolution as combo box data so we don't have to 
		// translate back from string to resolution when the user selects one.
		m_resolutionCombo.SetItemData(j, prop.list.items.piVal[j]);	
		
		// Remember which index is the current selection.
		if(prop.list.items.piVal[j] == prop.list.current.iVal)
		{
			currentSelection = j;
		}
	}

	m_resolutionCombo.SetCurSel(currentSelection);

	// Check if the scanner has duplex capability.
	prop.propertyID = SIP_DUPLEX_ENABLED;
	result = m_scanner.SI_GetProperty(&prop);

	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));

		m_duplexCapable = true;		// used to re-enable the checkbox after a scan

	}
	else 
	{
		// The only other result we should get is that the duplex property is not supported.
		assert(result == SIR_PROPERTY_UNSUPPORTED);

		m_duplexCapable = false;	// used to re-enable the checkbox after a scan
	}

	// Check if the scanner has end-of-page-detect capability.
	prop.propertyID = SIP_EOP_DETECT_ENABLED;
	result = m_scanner.SI_GetProperty(&prop);

	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));

		m_eopdCapable = true;		// used to re-enable the checkbox after a scan

	}
	else
	{
		// The only other result we should get is that the eop-detect property is not supported.
		assert(result == SIR_PROPERTY_UNSUPPORTED);

		m_eopdCapable = false;	// used to re-enable the checkbox after a scan
	}

	return true;
}

void ScanDialog::OnBnClickedCalibrateButton()
{

	SIResult result;
	if(!m_scannerIsOpen)
	{
		return;
	}

	// Check the scanner status
	SIScannerStatus	scannerStatus;
	result = m_scanner.SI_GetScannerStatus(&scannerStatus);
	
	if((result != SIR_SUCCESS) || (scannerStatus == SI_SS_OFFLINE))
	{
		CloseScanner();
		MessageBox("Scanner is not ready.");
		return;
	}


	EnableUIControls(FALSE);
	
	BeginWaitCursor();

	result = m_scanner.SI_Calibrate(SI_CT_DEFAULT, NULL);
	
	EndWaitCursor();

	if(result == SIR_SUCCESS)
	{
		MessageBox("The scanner was calibrated successfully.");
	}
	else
	{
		char *pErrorText;
		m_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}

	EnableUIControls(TRUE);
}

void ScanDialog::OnBnClickedScanButton()
{
	string str;	// for misc. use
	SIResult result;
	
	if(!m_scannerIsOpen)
	{
		return;
	}

	// Check the scanner status
	SIScannerStatus	scannerStatus;
	result = m_scanner.SI_GetScannerStatus(&scannerStatus);
	
	if((result != SIR_SUCCESS) || (scannerStatus == SI_SS_OFFLINE))
	{
		CloseScanner();
		MessageBox("Scanner is not ready.");
		return;
	}

	// Ensure paper is in
	SIPaperStatus paperStatus = SI_PS_PAPER_OUT;
	int mbResult;		// message box result.

	while(paperStatus == SI_PS_PAPER_OUT)
	{
		m_scanner.SI_GetPaperStatus(&paperStatus);

		if(paperStatus == SI_PS_PAPER_OUT)
		{
			mbResult = MessageBox("Please insert paper and press OK, or press Cancel to cancel the scan.",
				"MiniScan", MB_OKCANCEL);
			
			if(mbResult == IDCANCEL)
			{
				m_scanner.SI_CloseInterface();
				m_scanner.Detach();
				return;
			}
		}
	};


	SetScannerProperties();


	// Allocate a line buffer. 
	// First, find out how many bytes per line the scanner driver will return.
	SIProperty prop;
	prop.propertyID = SIP_LINE_WIDTH_IN_BYTES;
	result = m_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));

	m_lineWidthInBytes = prop.single.current.iVal;

	// round up to the nearest 4-byte boundary for Windows bitmap format.
	int lineWidthInBytesAligned = ((m_lineWidthInBytes + 3) / 4) * 4;

	m_linesPerBuffer = BUFFER_SIZE_IN_BYTES / lineWidthInBytesAligned;

	
	// Retieve the actual bitsPerPixel, width, and length so we can set up the 
	// bitmap file parameters.
	prop.propertyID = SIP_BITS_PER_PIXEL;
	result = m_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
	m_bitsPerPixel = prop.single.current.iVal;

	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	result = m_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
	m_scanWidthInPixels = prop.range.current.iVal;

	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	result = m_scanner.SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
	m_scanLength = prop.range.current.iVal;


	((CMainFrame *)GetParent())->SetBitmapFilename(0, "");
	((CMainFrame *)GetParent())->SetBitmapFilename(1, "");
	
	m_numSidesToRead = 1;

	// If the duplex property exists and if it is enabled, read image
	// data for the back also.
	prop.propertyID = SIP_DUPLEX_ENABLED;
	result = m_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));
		if(prop.single.current.bVal == SI_TRUE)
		{
			m_numSidesToRead++;
		}
	}


	// Disable controls while scanning.
	EnableUIControls(FALSE);

	result = m_scanner.SI_StartScan();

	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		m_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);

		if(result != SIR_SCANNER_BUSY)
		{
			m_scanner.SI_CloseInterface();
			m_scanner.Detach();
		}
		EnableUIControls(TRUE);
		return;
	}

	m_scanState = SS_INIT;
	m_currentReadSide = 0;

}

void ScanDialog::SetScannerProperties()
{
	// retrieve scan mode and resolution
	int32 scanMode;
	int32 resolution;

	SIResult result;

	scanMode = (int32)m_modeCombo.GetItemData(m_modeCombo.GetCurSel());
	resolution = (int32)m_resolutionCombo.GetItemData(m_resolutionCombo.GetCurSel());

	// set the scanner properties for mode and resolution
	SIProperty prop;
	prop.propertyID = SIP_SCAN_MODE;
	prop.containerType = SICON_LIST;
	prop.itemType = SI_INT32;
	prop.list.current.iVal = scanMode;
	result = m_scanner.SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);
	
	prop.propertyID = SIP_XRESOLUTION;
	prop.containerType = SICON_LIST;
	prop.itemType = SI_INT32;
	prop.list.current.iVal = resolution;
	result = m_scanner.SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);

	// set scan width to maximum width.
	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	result = m_scanner.SI_GetProperty(&prop);
	assert((result == SIR_SUCCESS) && (prop.containerType==SICON_RANGE) && (prop.itemType == SI_INT32));
	prop.range.current.iVal = prop.range.maximum.iVal;
	result = m_scanner.SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);

	// Set scan length to the default value. This will be different for A6 and A4 scanners.
	// (Using the default length value may not work correctly for scanner drivers 
	//  released before April 11, 2006 due to a bug in the length property's default value.)
	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	result = m_scanner.SI_GetProperty(&prop);
	assert((result == SIR_SUCCESS) && (prop.containerType==SICON_RANGE) && (prop.itemType == SI_INT32));
	prop.range.current.iVal = prop.range.def.iVal;
	result = m_scanner.SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);


	prop.propertyID = SIP_CHANNEL_ORDER;
	result = m_scanner.SI_GetProperty(&prop);
	assert((result == SIR_SUCCESS) && (prop.containerType==SICON_LIST) && (prop.itemType == SI_INT32));
	prop.list.current.iVal = SI_CO_BGR;
	result = m_scanner.SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);


	int32 duplex = (m_duplexCheckbox.GetCheck() == TRUE) ? SI_TRUE : SI_FALSE;

	prop.propertyID = SIP_DUPLEX_ENABLED;
	result = m_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		prop.single.current.iVal = duplex;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);
	}

	int32 eopDetect = (m_eopdCheckbox.GetCheck() == TRUE) ? SI_TRUE : SI_FALSE;

	prop.propertyID = SIP_EOP_DETECT_ENABLED;
	result = m_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		prop.single.current.iVal = eopDetect;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);
	}

}

void ScanDialog::OnTimer(UINT_PTR nIDEvent)
{
	SIResult result;
	
	if(m_scanner.IsAttached() && (m_scanState == SS_IDLE))
	{
		if(m_scannerIsOpen)
		{

			// Check the scanner status
			SIScannerStatus	scannerStatus;
			result = m_scanner.SI_GetScannerStatus(&scannerStatus);
	
			if(	(result == SIR_SUCCESS) && 
				((scannerStatus == SI_SS_ONLINE_READY) ||(scannerStatus == SI_SS_ONLINE_BUSY)) 
				)
			{
				CheckForPrefeed();
			}
			else
			{
				CloseScanner();
				SetStatusText("Scanner is not connected.");

				return;
			}
		}
		else
		{
			// attempt to open the scanner
			OpenScanner();
		}
	}

	DialogBarEx::OnTimer(nIDEvent);
}


bool ScanDialog::CheckForPrefeed()
{
	const float32 INCHES_TO_FEED = 0.125f;
	const int PREFEED_DELAY_IN_MS = 1000;

	bool shouldDetach = false;
	bool shouldClose = false;

	if(m_scanState != SS_IDLE)
	{
		return true;
	}

	if(!m_prefeedEnabled)
	{
		return true;
	}

	SIPaperStatus paperStatus;
	SIResult result;

	result = m_scanner.SI_GetPaperStatus(&paperStatus);

	if(result != SIR_SUCCESS)
	{
		CloseScanner();
		return true;
	}

	if((paperStatus == SI_PS_PAPER_IN) && (m_prevPaperStatus == SI_PS_PAPER_OUT))
	{
		// Provide a delay for the time it takes the user to move the paper 
		// past the paper sensor to the roller.
		Sleep(PREFEED_DELAY_IN_MS);

		if(m_scanState == SS_IDLE)
		{
	
		SIProperty prop;
		prop.propertyID = SIP_OPTICAL_RESOLUTION;
		result = m_scanner.SI_GetProperty(&prop);				// call to get the correct item type
		assert(result == SIR_SUCCESS);
		assert(prop.containerType == SICON_SINGLE && prop.itemType == SI_INT32);
		uint32 opticalResolution = prop.single.current.iVal;
		assert(opticalResolution != 0 && opticalResolution < 5000);

		// Set Y resolution as the optical resolution
		int32 saveYresolution;
		prop.propertyID = SIP_YRESOLUTION;
		result = m_scanner.SI_GetProperty(&prop);				// call to get the correct item type
		assert(result == SIR_SUCCESS);
		assert(prop.containerType == SICON_LIST && prop.itemType == SI_INT32);
		saveYresolution = prop.list.current.iVal;
		prop.list.current.iVal = opticalResolution;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);

		// Set Y offset to the distance to feed
		int32 saveYoffset;
		prop.propertyID = SIP_YOFFSET;
		result = m_scanner.SI_GetProperty(&prop);				// call to get the correct item type
		assert(result == SIR_SUCCESS);
		assert(prop.containerType == SICON_RANGE && prop.itemType == SI_INT32);
		saveYoffset = prop.range.current.iVal;
		prop.containerType = SICON_SINGLE;
		prop.single.current.iVal = uint32(INCHES_TO_FEED * opticalResolution); // offset in lines
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);


		// Save feed direction and set forward
		int32 saveFeedDirection;
		prop.propertyID = SIP_FEED_DIRECTION;
		result = m_scanner.SI_GetProperty(&prop);				// call to get the correct item type
		assert(result == SIR_SUCCESS);
		assert(prop.containerType == SICON_LIST && prop.itemType == SI_INT32);
		saveFeedDirection = prop.list.current.iVal;
		prop.containerType = SICON_SINGLE;
		prop.single.current.iVal = SI_FORWARD;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);

		// If the SIP_FEED_RATE property is supported for this scanner, set the feeding 
		// speed to a good value for prefeeding.
		float32 saveFeedRate;
		prop.propertyID = SIP_FEED_RATE;				// note that this property may be unsupported
		result = m_scanner.SI_GetProperty(&prop);		// call to get the correct container and item type
		if(result == SIR_SUCCESS)
		{
			saveFeedRate = prop.range.current.fVal;	// save the old value
			prop.range.current.fVal = 1.37f;		// 1.37 inches per second is a good speed...
			
			// ...but limit this to available speeds.
			prop.range.current.fVal = max( min(prop.range.current.fVal, prop.range.maximum.fVal), 
											prop.range.minimum.fVal);
			m_scanner.SI_SetProperty(&prop);
		}

		// Do the prefeed
		m_scanner.SI_Feed();

		prop.propertyID = SIP_YRESOLUTION;
		prop.containerType = SICON_SINGLE;
		prop.itemType = SI_INT32;
		prop.single.current.iVal = saveYresolution;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);

		// Restore Y offset
		prop.propertyID = SIP_YOFFSET;
		prop.containerType = SICON_SINGLE;
		prop.itemType = SI_INT32;
		prop.single.current.iVal = saveYoffset;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);

		// Restore feed direction
		prop.propertyID = SIP_FEED_DIRECTION;
		prop.containerType = SICON_SINGLE;
		prop.itemType = SI_INT32;
		prop.single.current.iVal = saveFeedDirection;
		result = m_scanner.SI_SetProperty(&prop);
		assert(result == SIR_SUCCESS);

		// Restore feed speed.
		prop.propertyID = SIP_FEED_RATE;
		prop.containerType = SICON_SINGLE;
		prop.itemType = SI_FLOAT32;
		prop.single.current.fVal = saveFeedRate;
		m_scanner.SI_SetProperty(&prop);	// This call may fail if SIP_FEED_RATE isn't supported. That's OK.

		}
	}

	m_prevPaperStatus = paperStatus;

	return true;
}


BOOL ScanDialog::OnIdle(LONG lCount)
{

	if(m_scanState == SS_IDLE)
	{
		return FALSE;
	}
	
	if(m_scanState == SS_INIT)
	{

		// Setup a bitmap file (and possibly a second one for the back) for the thread 
		// to write image data into.
		m_bitmapFile[m_currentReadSide].Open(m_filename[m_currentReadSide]);
		m_bitmapFile[m_currentReadSide].SetBitsPerPixel(m_bitsPerPixel);
		m_bitmapFile[m_currentReadSide].SetWidth(m_scanWidthInPixels);
		m_bitmapFile[m_currentReadSide].SetHeight(-m_scanLength);

		m_totalLinesLeftToRead = m_scanLength;
		m_totalLinesRead = 0;
	
		m_scanState = SS_READ;
	}
	
	if(m_scanState == SS_READ)
	{

		if(m_totalLinesLeftToRead > 0)
		{
			SIResult result;
			uint32 linesReturned;
	
			int linesToRead = min(m_linesPerBuffer, m_totalLinesLeftToRead);

			// Get image data from the scanner
			result = m_scanner.SI_ReadImageData(m_lineBuffer, linesToRead, m_currentReadSide, &linesReturned);

			if(result == SIR_SUCCESS)
			{
				// Align the data to a 4-byte boundary
				int bytesToWrite = AlignInPlace(m_lineBuffer, linesReturned, m_lineWidthInBytes);

				// Write the image data out to the file
				m_bitmapFile[m_currentReadSide].Write((char *)(m_lineBuffer), bytesToWrite);

				m_totalLinesRead += linesReturned;
				m_totalLinesLeftToRead -= linesReturned;

			}
			else
			{
				if(result == SIR_ENDOFDATA)
				{
					// If we get SIR_ENDOFDATA, then one of three things happened:
					//   1. End-of-page detection was enabled and this is the end. There are not more lines.
					//   2. We tried to read more lines that we specified in the scan parameters.
					//      (This shouldn't happen if you only read as many as you specify.)
					//   3. The scanner was disconnected before all the expected data was scanned.
					TRACE2("Received SIR_ENDOFDATA for side %d. totalLinesRead = %d\n", m_currentReadSide, m_totalLinesRead);
					m_scanState = ((m_currentReadSide + 1) < m_numSidesToRead) ? SS_INIT : SS_END;
				}
				else	// an error
				{
					char *pErrorText;
					m_scanner.SI_GetLastErrorText(&pErrorText);
					MessageBox(pErrorText);
				
					m_scanState = SS_END;
				}

				// In any case that is not SIR_SUCCESS, update the bitmap file length 
				// and close the file.
				m_bitmapFile[m_currentReadSide].SetHeight(-m_totalLinesRead);
				m_bitmapFile[m_currentReadSide].Close();

				((CMainFrame *)GetParent())->SetBitmapFilename(m_currentReadSide, m_filename[m_currentReadSide]);

				m_currentReadSide++;	// continue to next side if this is a duplex scan
				m_totalLinesLeftToRead = 0;
				
			}

		}
		else	// done reading all lines
		{
			// At this point we've read all the lines in the image.
			// So update the bitmap file length and close the file.
			m_bitmapFile[m_currentReadSide].SetHeight(-m_totalLinesRead);
			m_bitmapFile[m_currentReadSide].Close();
				
			((CMainFrame *)GetParent())->SetBitmapFilename(m_currentReadSide, m_filename[m_currentReadSide]);
			
			m_currentReadSide++;	// continue to next side if this is a duplex scan
			m_scanState = (m_currentReadSide < m_numSidesToRead) ? SS_INIT : SS_END;

		}
	}

	if(m_scanState == SS_CANCEL)
	{
		m_bitmapFile[m_currentReadSide].SetHeight(-m_totalLinesRead);
		m_bitmapFile[m_currentReadSide].Close();

		((CMainFrame *)GetParent())->SetBitmapFilename(m_currentReadSide, m_filename[0]);

		m_scanState = SS_END;
	}

	if(m_scanState == SS_END)
	{

		// call SI_EndScan
		SIResult result = m_scanner.SI_EndScan();

		if(result != SIR_SUCCESS)
		{
			char *pErrorText;
			m_scanner.SI_GetLastErrorText(&pErrorText);
			MessageBox(pErrorText);
		}
		
		result = m_scanner.SI_FeedPaperOut();

		if(result != SIR_SUCCESS)
		{
			char *pErrorText;
			m_scanner.SI_GetLastErrorText(&pErrorText);
			MessageBox(pErrorText);
		}

		m_scanState = SS_IDLE;
		
		// Re-enable controls while scanning.
		EnableUIControls();
	}

	return TRUE; // return TRUE to get more processing time
}

// returns the number of bytes in the buffer after alignment.
int ScanDialog::AlignInPlace(unsigned char* pBuffer, int numberOfLines, int bytesPerLine)
{
	if(bytesPerLine % 4)
	{
		int bytesPerLineAligned = ((bytesPerLine + 3) / 4) * 4;

		int padBytes = bytesPerLineAligned - bytesPerLine ;

		unsigned char* pSrc = pBuffer + (bytesPerLine * numberOfLines) - 1;
		unsigned char* pDst = pBuffer + (bytesPerLineAligned * numberOfLines) - 1;

		while (pSrc >= pBuffer)
		{
			pDst -= padBytes;

			for(int i = 0; i < bytesPerLine; i++)
			{
				*pDst-- = *pSrc--;
			}
		}

		return bytesPerLineAligned * numberOfLines;
	}

	return bytesPerLine * numberOfLines;
}

void ScanDialog::PostNcDestroy()
{

	if(m_lineBuffer)
	{
		delete [] m_lineBuffer;
		m_lineBuffer = NULL;
	}

	DialogBarEx::PostNcDestroy();
}


void ScanDialog::SetStatusText(string text)
{
	((CMainFrame*)GetParent())->SetStatusText(text);
}

void ScanDialog::EnablePrefeed(bool enable)
{
	m_prefeedEnabled = enable;
}

