#pragma once

#include "DialogBarEx.h"
#include "Scanner.h"
#include "BitmapFile.h"

// ScanDialog dialog

class ScanDialog : public DialogBarEx
{
	DECLARE_DYNAMIC(ScanDialog)

	Scanner		m_scanner;

	enum { BUFFER_SIZE_IN_BYTES = 0x8000 };
	unsigned char*	m_lineBuffer;
	int				m_lineWidthInBytes;
	int				m_linesPerBuffer;
	int				m_scanLength;
	int				m_bitsPerPixel;
	int				m_scanWidthInPixels;
	BitmapFile		m_bitmapFile[2];
	CString			m_filename[2];

	enum {	SS_IDLE, 
			SS_INIT, 
			SS_READ, 
			SS_CANCEL,
			SS_END
	} m_scanState;
	int		m_currentReadSide;
	int		m_numSidesToRead;
	int		m_totalLinesLeftToRead;
	int		m_totalLinesRead;
	bool	m_duplexCapable;
	bool	m_eopdCapable;
	bool	m_endScan;
	bool	m_scannerIsOpen;
	
	SIPaperStatus	m_prevPaperStatus;

	bool OpenScanner();
	void CloseScanner();
	bool GetScannerProperties();
	void SetScannerProperties();
	int	AlignInPlace(unsigned char* pBuffer, int numberOfLines, int bytesPerLine);
	void SetStatusText(string text);
	void EnablePrefeed(bool enable = true);

public:
	ScanDialog(CWnd* pParent = NULL);   // standard constructor
	virtual ~ScanDialog();
	virtual BOOL OnInitDialogBar();
	BOOL OnIdle(LONG lCount);

// Dialog Data
	enum { IDD = IDD_SCAN_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	CComboBox	m_scannerNameCombo;
	CComboBox	m_modeCombo;
	CComboBox	m_resolutionCombo;
	CButton		m_duplexCheckbox;
	CButton		m_eopdCheckbox;
	CButton		m_scanButton;
	CButton		m_calibrateButton;

	bool m_prefeedEnabled;

	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnScannerComboSelectChanged();
	afx_msg void OnBnClickedScanButton();
	afx_msg void OnBnClickedCalibrateButton();
protected:
	virtual void PostNcDestroy();

	bool CheckForPrefeed();
	void EnableUIControls(BOOL enable = TRUE);


public:
	afx_msg void OnTimer(UINT_PTR nIDEvent);
};

