
#include "StdAfx.h"
#include "Scanner.h"
#include "SIScanner.h"
#include <cassert>

Scanner::Scanner()
	:m_pScanner(NULL)
{


}

Scanner::~Scanner(void)
{
	if(IsAttached())
	{
		Detach();
	}
}

/*

	Note that although this function is virtual, a ca;; from the application  
	will always come here. This is an envelope class and the application doesn't 
	have a pointer to the real (letter) scanner object. So there is not danger 
	of the application calling Attach() directly.
*/
bool Scanner::Attach(string DllPathname)
{

	if(m_pScanner != NULL)
	{
		// If the DLL filename is the same as one we already have created,
		// then don't delete and reallocate a new scanner object, since we know 
		// we have the correct type. Instead just pass on the call.
		if(0 != m_DllPathname.compare(DllPathname))
		{
			// Get rid of existing scanner DLL.
			if(m_pScanner->IsAttached())
			{
				m_pScanner->Detach();
			}
		}
	}

	// At this point, the only way the m_pScanner is not NULL is if the correct
	// scanner DLL type is already allocated. If it is NULL, allocate a new one.
	if(m_pScanner == NULL)
	{

		// Determine which type of scanner interface to create based on the DLL filename
		//
		//
		m_pScanner = new SIScanner();
	
		if(m_pScanner == NULL)
		{
			assert(false);
			return false;
		}
	}

	bool result =  m_pScanner->Attach(DllPathname);

	if(!result)
	{
		delete m_pScanner;
		m_pScanner = NULL;
		return false;
	}

	return true;
}


bool Scanner::IsAttached()
{
	if(m_pScanner)
	{
		return m_pScanner->IsAttached();
	}
	
	return false;
}

void  Scanner::Detach()
{
	if(m_pScanner)
	{
		m_pScanner->Detach();
	}

	delete m_pScanner;
	m_pScanner = NULL;

}


SIResult Scanner::SI_GetScannerStatus(SIScannerStatus *pScannerStatus)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetScannerStatus(pScannerStatus) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_GetPaperStatus(SIPaperStatus *pPaperStatus)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetPaperStatus(pPaperStatus) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_GetButtonStatus(uint32 buttonNumber, SIButtonStatus *pButtonStatus)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetButtonStatus(buttonNumber, pButtonStatus) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_GetProperty(SIProperty* pProperty)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetProperty(pProperty) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_SetProperty(SIProperty* pProperty)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_SetProperty(pProperty) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_Reset()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_Reset() :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_Diagnostic(SIDiagTest test, void* pDiagInfo)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_Diagnostic(test, pDiagInfo) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_GetLastError()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetLastError() :
		SIR_UNKNOWN_ERROR;

}

SIResult Scanner::SI_GetLastErrorText(char **ppErrorText)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetLastErrorText(ppErrorText) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_StartScan()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_StartScan() :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_ReadImageData(uint8 *buffer, uint32 numberOfLinesToRead, uint32 pageNumber, uint32* numberOfLinesReturned)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_ReadImageData(buffer, numberOfLinesToRead, pageNumber, numberOfLinesReturned) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_EndScan()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_EndScan() :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_OpenInterface(const char* pName)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_OpenInterface(pName) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_CloseInterface()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_CloseInterface() :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_GetEvent(SIEventInfo* pEventInfo)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_GetEvent(pEventInfo) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_Calibrate(uint32 target, ProgressCallbackPtr pProgressCallback)
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_Calibrate(target, pProgressCallback) :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_IsCalibrated()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_IsCalibrated() :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_FeedPaperOut()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_FeedPaperOut() :
		SIR_UNKNOWN_ERROR;
}

SIResult Scanner::SI_Feed()
{
	assert(m_pScanner);
	return m_pScanner ? 
		m_pScanner->SI_Feed() :
		SIR_UNKNOWN_ERROR;
}

	