#if !defined(SCANNER_H)
#define SCANNER_H

#include "ScannerAPI.h" // the project needs to point to the correct directory to access this
#include <string>
using namespace std;


/*
	The Scanner class is a base class for communicating with the scanner using 
	the "SI_" interface. Using a heirarchy here may at first seem overkill for 
	a program as simple as this MiniScan utility. However, it's also meant to  
	be used with another existing API interface which is also derived from 
	the Scanner class. The other interface is not included here.
*/

class Scanner
{
public:
	Scanner();
	virtual ~Scanner(void);

	virtual bool Attach(string DllPathname);
	virtual bool IsAttached();
	virtual void Detach(void);


	virtual SIResult SI_GetScannerStatus(SIScannerStatus *pScannerStatus);
	virtual SIResult SI_GetPaperStatus(SIPaperStatus *pPaperStatus);
	virtual SIResult SI_GetButtonStatus(uint32 buttonNumber, SIButtonStatus *pButtonStatus);
	virtual SIResult SI_GetProperty(SIProperty* pProperty);
	virtual SIResult SI_SetProperty(SIProperty* pProperty);
	virtual SIResult SI_Reset();
	virtual SIResult SI_Diagnostic(SIDiagTest test, void* pDiagInfo);
	virtual SIResult SI_GetLastError();
	virtual SIResult SI_GetLastErrorText(char **ppErrorText);
	virtual SIResult SI_StartScan();
	virtual SIResult SI_ReadImageData(uint8 *buffer, uint32 numberOfLinesToRead, uint32 pageNumber, uint32* numberOfLinesReturned);
	virtual SIResult SI_EndScan();
	virtual SIResult SI_OpenInterface(const char* pName);
	virtual SIResult SI_CloseInterface();
	virtual SIResult SI_GetEvent(SIEventInfo* pEventInfo);
	virtual SIResult SI_Calibrate(uint32 target, ProgressCallbackPtr pProgressCallback);
	virtual SIResult SI_IsCalibrated();
	virtual SIResult SI_FeedPaperOut();
	virtual SIResult SI_Feed();


private:
	Scanner*	m_pScanner;
	string		m_DllPathname;

};
#endif