// This is the main DLL file.

#include "stdafx.h"

#include "Scanner.h"

using namespace System;
using namespace System::Runtime::InteropServices;


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif


namespace ScannerMng
{

	// This dummy definition of array is needed to alert the compiler that we're going 
	// to use the word 'array' as one of our variable names instead of a keyword. 
	// (The SIProperty structure uses this.) So from this point on, if we REALLY want 
	// to consruct a managed array, we need to explicitly qualify it as coming from 
	// the cli namespace, as in:    "cli::array"
	// For more info, check the MSDN2 documentation under "cli namespace".
	int array = 0;

	// In earlier versions of this project, the fields after the first three were placed at 
	// offset 4. This was changed to offset 8 to support 64-bit compilation and to be consistent 
	// between 32-bit and 64-bit. Therefore, you may need to recompile your existing .NET 
	// application to work with this DLL on 32-bit systems. 
	[StructLayout(LayoutKind::Explicit)]
	public ref class SIValue
	{
	public:
		[FieldOffset(0)]			Int32	iVal;		///< integer value (SI_INT32)
		[FieldOffset(0)]			Single	fVal;		///< float value (SI_FLOAT32)
		[FieldOffset(0)]			Int32	bVal;		///< boolean value. (SI_BOOL. Either SI_TRUE or SI_FALSE)
		[FieldOffset(8)]	String^	strVal;				///< string value (SI_STR)

		// The following are used only with array or list containers.
		[FieldOffset(8)]	cli::array<System::Int32>^		piVal;		///< pointer to integer (SI_INT32)
		[FieldOffset(8)]	cli::array<System::Int16>^		psVal;		///< pointer to short (SI_INT16)
		[FieldOffset(8)]	cli::array<System::SByte>^		pcVal;		///< pointer to character (SI_INT8)
		[FieldOffset(8)]	cli::array<System::UInt32>^		puiVal;		///< pointer to unsigned integer (SI_UINT32)
		[FieldOffset(8)]	cli::array<System::UInt16>^		pusVal;		///< pointer to unsigned short (SI_UINT16)
		[FieldOffset(8)]	cli::array<System::Byte>^		pucVal;		///< pointer to unsigned character (SI_UINT8)
		[FieldOffset(8)]	cli::array<System::String^>^	pstrVal;	///< pointer to strings (SI_STR)
		[FieldOffset(8)]	cli::array<System::Boolean>^	pbVal;		///< pointer to boolean (SI_BOOL)
		[FieldOffset(8)]	cli::array<System::Single>^		pfVal;		///< pointer to float (SI_FLOAT32)

	} ;


	public ref class SISingle
	{
	public:
		SIValue^	current;
		SIValue^	def;

		SISingle()
		{
			current = gcnew SIValue();
			def = gcnew SIValue();
		}
	};


	public ref class SIRange
	{
	public:
		SIValue^	minimum;
		SIValue^	maximum;
		SIValue^	stepSize;
		SIValue^	def;
		SIValue^	current;

		SIRange()
		{
			current = gcnew SIValue();
			def = gcnew SIValue();
			stepSize = gcnew SIValue();
			minimum = gcnew SIValue();
			maximum = gcnew SIValue();
		}
	};



	public ref class SIList
	{
	public:
		Int32		numItems;
		SIValue^	current;
		SIValue^	def;
		SIValue^	items;

		SIList()
		{
			numItems = 0;
			current = gcnew SIValue();
			def = gcnew SIValue();
			items = gcnew SIValue();
		}
	};

	public ref class SIArray
	{
	public:
		Int32	numItems;
		SIValue^	items;

		SIArray()
		{
			numItems = 0;
			items = gcnew SIValue();
		}
	};

	[StructLayout(LayoutKind::Explicit)]
	public ref class SIProperty
	{
	public:
		[FieldOffset(0)]	UInt16		propertyID;			///< a property ID constant.
		[FieldOffset(2)]	UInt16		containerType;		///< a container type constant.
		[FieldOffset(4)]	UInt16		itemType;			///< the data type the container holds.
		[FieldOffset(6)]	UInt16		access;				///< Set to SIACC_READWRITE if this property can be changed. 
															///< Set to SIACC_READONLY if this property is read-only.
		// containers
		[FieldOffset(8)]	SISingle^	single;
		[FieldOffset(8)]	SIRange^	range;
		[FieldOffset(8)]	SIList^		list;
		[FieldOffset(8)]	SIArray^	array;

	static const UInt16	SIP_INVALID =					SINative::SIP_INVALID;
	static const UInt16	SIP_OPTICAL_RESOLUTION =		SINative::SIP_OPTICAL_RESOLUTION;
	static const UInt16 SIP_OPTICAL_WIDTH_IN_PIXELS =	SINative::SIP_OPTICAL_WIDTH_IN_PIXELS;
	static const UInt16 SIP_XRESOLUTION =				SINative::SIP_XRESOLUTION;
	static const UInt16 SIP_YRESOLUTION =				SINative::SIP_YRESOLUTION;
	static const UInt16 SIP_BITS_PER_CHANNEL =			SINative::SIP_BITS_PER_CHANNEL;
	static const UInt16 SIP_BITS_PER_PIXEL =			SINative::SIP_BITS_PER_PIXEL;
	static const UInt16 SIP_SCAN_MODE =					SINative::SIP_SCAN_MODE;
	static const UInt16 SIP_BRIGHTNESS =				SINative::SIP_BRIGHTNESS;
	static const UInt16 SIP_CONTRAST =					SINative::SIP_CONTRAST;
	static const UInt16 SIP_PHOTOMETRIC_INTERPRETATION = SINative::SIP_PHOTOMETRIC_INTERPRETATION;
	static const UInt16 SIP_YOFFSET =					SINative::SIP_YOFFSET;
	static const UInt16 SIP_XOFFSET =					SINative::SIP_XOFFSET;
	static const UInt16 SIP_SCAN_WIDTH_IN_PIXELS =		SINative::SIP_SCAN_WIDTH_IN_PIXELS;
	static const UInt16 SIP_SCAN_LENGTH_IN_LINES =		SINative::SIP_SCAN_LENGTH_IN_LINES;
	static const UInt16 SIP_PLANARCHUNKY =				SINative::SIP_PLANARCHUNKY;
	static const UInt16 SIP_CHANNEL_ORDER =				SINative::SIP_CHANNEL_ORDER;
	static const UInt16 SIP_MAX_SCAN_TIME_IN_SEC =		SINative::SIP_MAX_SCAN_TIME_IN_SEC;
	static const UInt16 SIP_LINE_WIDTH_IN_BYTES =		SINative::SIP_LINE_WIDTH_IN_BYTES;
	static const UInt16 SIP_DUPLEX_ENABLED =			SINative::SIP_DUPLEX_ENABLED;
	static const UInt16 SIP_SPOOLER_ENABLED =			SINative::SIP_SPOOLER_ENABLED;
	static const UInt16 SIP_FEED_DIRECTION =			SINative::SIP_FEED_DIRECTION;
	static const UInt16 SIP_OPTICAL_LENGTH_IN_PIXELS =	SINative::SIP_OPTICAL_LENGTH_IN_PIXELS;
	static const UInt16 SIP_GAMMA =						SINative::SIP_GAMMA;
	static const UInt16 SIP_HIGHLIGHT =					SINative::SIP_HIGHLIGHT;
	static const UInt16 SIP_SHADOW =					SINative::SIP_SHADOW;
	static const UInt16 SIP_LUT_GRAY =					SINative::SIP_LUT_GRAY;
	static const UInt16 SIP_LUT_RED =					SINative::SIP_LUT_RED;
	static const UInt16 SIP_LUT_GREEN =					SINative::SIP_LUT_GREEN;
	static const UInt16 SIP_LUT_BLUE =					SINative::SIP_LUT_BLUE;
	static const UInt16 SIP_THRESHOLD =					SINative::SIP_THRESHOLD;
	static const UInt16 SIP_EOP_DETECT_ENABLED =		SINative::SIP_EOP_DETECT_ENABLED;
	static const UInt16 SIP_DESCREEN_ENABLED =			SINative::SIP_DESCREEN_ENABLED;
	static const UInt16 SIP_DROPOUT_COLOR =				SINative::SIP_DROPOUT_COLOR;
	static const UInt16 SIP_USB_RATE =					SINative::SIP_USB_RATE;
	static const UInt16 SIP_LED_INDICATOR1 =			SINative::SIP_LED_INDICATOR1;
	static const UInt16 SIP_LED_INDICATOR2 =			SINative::SIP_LED_INDICATOR2;
	static const UInt16 SIP_FEED_RATE =					SINative::SIP_FEED_RATE;
	static const UInt16 SIP_SCAN_RATE =					SINative::SIP_SCAN_RATE;
	static const UInt16 SIP_DIRECTORY_PATH = 200;

	// Container types
	static const UInt16 SICON_SINGLE =		SINative::SICON_SINGLE;
	static const UInt16 SICON_RANGE =		SINative::SICON_RANGE;
	static const UInt16 SICON_LIST =		SINative::SICON_LIST;
	static const UInt16 SICON_ARRAY =		SINative::SICON_ARRAY;

	// Item types
	static const UInt16 SI_INT32 =		SINative::SI_INT32;		///< 32-bit signed integer
	static const UInt16 SI_UINT32 =		SINative::SI_UINT32;		///< 32-bit unsigned integer
	static const UInt16 SI_INT16 =		SINative::SI_INT16;		///< 16-bit signed integer
	static const UInt16 SI_UINT16 =		SINative::SI_UINT16;		///< 16-bit unsigned integer
	static const UInt16 SI_INT8 =		SINative::SI_INT8;		///< 8-bit signed integer
	static const UInt16 SI_UINT8 =		SINative::SI_UINT8;		///< 8-bit unsigned integer
	static const UInt16 SI_BOOL =		SINative::SI_BOOL;		///< The item will be either SI_TRUE or SI_FALSE.
	static const UInt16 SI_FLOAT32 =	SINative::SI_FLOAT32;		///< single-precision floating point 
	static const UInt16 SI_STR =		SINative::SI_STR;		///< string. The item will be a pointer to an ANSI zero-terminated string.

	static const UInt16 SIACC_READONLY =	SINative::SIACC_READONLY;
	static const UInt16 SIACC_READWRITE =	SINative::SIACC_READWRITE;

	static const Int32 SI_SCANMODE_BW =		SINative::SI_SCANMODE_BW;
	static const Int32 SI_SCANMODE_GRAY =	SINative::SI_SCANMODE_GRAY;
	static const Int32 SI_SCANMODE_RGB =	SINative::SI_SCANMODE_RGB;

	static const Int32 SI_PI_BLACKZERO =	SINative::SI_PI_BLACKZERO;	///< Photometric Interpretation. Black is represented by zero
	static const Int32 SI_PI_WHITEZERO =	SINative::SI_PI_WHITEZERO;	///< Photometric Interpretation. White is represented by zero

	static const Int32 SI_PC_PLANAR =		SINative::SI_PC_PLANAR;
	static const Int32 SI_PC_CHUNKY =		SINative::SI_PC_CHUNKY;

	static const Int32 SI_CO_RGB =			SINative::SI_CO_RGB;	///< Value of SIP_CHANNEL_ORDER. SI_CO_RGB specifies that pixels are in red-green-blue order.
	static const Int32 SI_CO_BGR =			SINative::SI_CO_BGR;	///< Value of SIP_CHANNEL_ORDER. SI_CO_BRG specifies that pixels are in blue-green-red order.

	static const Int32 SI_DO_NONE =			SINative::SI_DO_NONE;	///< Value of SIP_DROPOUT_COLOR. SI_DO_NONE means no dropout.
	static const Int32 SI_DO_RED =			SINative::SI_DO_RED;	///< Value of SIP_DROPOUT_COLOR. SI_DO_RED means red dropout.
	static const Int32 SI_DO_GREEN =		SINative::SI_DO_GREEN;	///< Value of SIP_DROPOUT_COLOR. SI_DO_GREEN means green dropout.
	static const Int32 SI_DO_BLUE =			SINative::SI_DO_BLUE;	///< Value of SIP_DROPOUT_COLOR. SI_DO_BLUE means blue dropout.

	static const UInt32 SI_FALSE =	SINative::SI_FALSE;
	static const UInt32 SI_TRUE =	SINative::SI_TRUE;

	// Motor feed directions. Used for SIP_FEED_DIRECTION.
	static const Int32 SI_FORWARD =	SINative::SI_FORWARD;
	static const Int32 SI_REVERSE =	SINative::SI_REVERSE;

	// These identifiers used with the SIP_USB_RATE property.
	static const Int32 SI_USB_FULL =	SINative::SI_USB_FULL;
	static const Int32 SI_USB_HIGH =	SINative::SI_USB_HIGH;

	};


	public enum class SICalibrationState :uint32
	{
		CALIBRATED = SINative::SI_FALSE,
		NOT_CALIBRATED = SINative::SI_TRUE
	};
	public enum class SICalibrationTarget
	{
		SI_CT_DEFAULT =		SINative::SI_CT_DEFAULT,
		SI_CT_WHITEONLY =	SINative::SI_CT_WHITEONLY,
	};

	public enum class SICallbackResult :Int32
	{
		CONTINUE = SINative::SI_TRUE,
		CANCEL = SINative::SI_FALSE,
	};
	public delegate SICallbackResult ProgressCallbackDelegate(UInt32 percentCompleted);

	public enum class SIScannerStatus
	{
		SI_SS_OFFLINE =			SINative::SI_SS_OFFLINE,
		SI_SS_ONLINE_READY =	SINative::SI_SS_ONLINE_READY,
		SI_SS_ONLINE_BUSY =		SINative::SI_SS_ONLINE_BUSY

	};

	public enum class SIButtonStatus
	{
		SI_BS_DOWN =		SINative::SI_BS_DOWN,
		SI_BS_UP =			SINative::SI_BS_UP
	};
	
	public enum class SIPaperStatus
	{
		SI_PS_PAPER_OUT =	SINative::SI_PS_PAPER_OUT,
		SI_PS_PAPER_IN =	SINative::SI_PS_PAPER_IN
	};

	public enum class SIDiagTest
	{
		SI_DIAG_HW_PRESENCE = SINative::SI_DIAG_HW_PRESENCE,
		SI_DIAG_READ_PERFORMANCE = SINative::SI_DIAG_READ_PERFORMANCE
	};

	public enum class SIDiagResult
	{
		SI_DIAG_FAIL = SINative::SI_DIAG_FAIL,
		SI_DIAG_PASS = SINative::SI_DIAG_PASS
	};
	
	public ref class SIDiagInfo abstract
	{
	};

	public ref class SIDiagHwPresenceInfo : public SIDiagInfo
	{
	public:
		UInt32 result;
	};

	public ref class SIDiagReadPerfInfo : public SIDiagInfo
	{
	public:
		UInt32 readLengthInKB;		//	[in]  Set this to the length of data to read from the hardware.
		UInt32 timeInMicroSec;		//	[out] Resulting time in microseconds will be returned here.
	};


	public ref class SIEventInfo
	{
	public:
		Byte						eventDataSize;	// [in] The size in bytes of the data in the eventData array.
		cli::array<System::Byte>^	eventData;		// [in] The data return by the scanner in the USB interrupt phase.
		UInt32						eventFlags;		// [out] Bit flags that indicate which event(s) occurred.
	};


	public enum class SIResult :uint32
	{
		SIR_SUCCESS =						SINative::SIR_SUCCESS,
		SIR_UNKNOWN_MODEL_NAME =			SINative::SIR_UNKNOWN_MODEL_NAME,
		SIR_SCANNER_NOT_READY =				SINative::SIR_SCANNER_NOT_READY,
		SIR_BAD_PARAMETER =					SINative::SIR_BAD_PARAMETER,
		SIR_ALREADY_OPEN =					SINative::SIR_ALREADY_OPEN,
		SIR_INTERFACE_NOT_OPEN =			SINative::SIR_INTERFACE_NOT_OPEN,
		SIR_FILESYSTEM_ACCESS_DENIED =		SINative::SIR_FILESYSTEM_ACCESS_DENIED,
		SIR_SCANNER_BUSY =					SINative::SIR_SCANNER_BUSY,
		SIR_ENDOFDATA =						SINative::SIR_ENDOFDATA,
		SIR_NOT_SCANNING =					SINative::SIR_NOT_SCANNING,
		SIR_DEVICE_COMMUNICATION_ERROR =	SINative::SIR_DEVICE_COMMUNICATION_ERROR,
		SIR_UNKNOWN_ERROR =					SINative::SIR_UNKNOWN_ERROR,
		SIR_NOT_CALIBRATED =				SINative::SIR_NOT_CALIBRATED,
		SIR_INITIALIZATION_FAILURE =		SINative::SIR_INITIALIZATION_FAILURE,
		SIR_USER_CANCELLED =				SINative::SIR_USER_CANCELLED,
		SIR_CALIB_WHITE_TARGET =			SINative::SIR_CALIB_WHITE_TARGET,
		SIR_PROPERTY_UNSUPPORTED =			SINative::SIR_PROPERTY_UNSUPPORTED,		///< The caller tried to set or retrieve a capabiity that is not supported in the current implementation,
		SIR_PROPERTY_INVALID_VALUE =		SINative::SIR_PROPERTY_INVALID_VALUE,	///< The caller tried to set a property that was outside of the valid range.
		SIR_PROPERTY_READONLY =				SINative::SIR_PROPERTY_READONLY,		///< The caller tried to set a property that has read-only access.
	};



public ref class SIScanner
{
public:

	//static const UInt32 SI_TRUE = SINative::SI_TRUE;
	//static const UInt32 SI_FALSE = SINative::SI_FALSE;

public:
	SIScanner(String^ dllFilename);

	SIResult SI_OpenInterface(String^ scannerName);
	SIResult SI_StartScan();
	SIResult SI_EndScan();
	SIResult SI_Feed();
	SIResult SI_FeedPaperOut();
	SIResult SI_Clean(ProgressCallbackDelegate^ progressCallback);
	SIResult SI_CloseInterface();
	SIResult SI_GetProperty(SIProperty^ prop); 
	SIResult SI_SetProperty(SIProperty^ prop); 
	SIResult SI_IsCalibrated(SICalibrationState %pState);
	SIResult SI_Calibrate(SICalibrationTarget calTarget, ProgressCallbackDelegate^ progressCallback);
	SIResult SI_GetPaperStatus(SIPaperStatus% paperStatus);
	SIResult SI_GetButtonStatus(UInt32 buttonNumber, SIButtonStatus% paperStatus);
	SIResult SI_GetScannerStatus(SIScannerStatus% scannerStatus);
	SIResult SI_ReadImageData(cli::array<Byte>^ buffer, UInt32 numberOfLinesToRead, UInt32 pageNumber, UInt32% numberOfLinesReturned );
	SIResult SI_Reset();
	SIResult SI_GetLastError();
	SIResult SI_GetLastErrorText(String^% errorText);
	SIResult SI_Diagnostic(SIDiagTest test, SIDiagInfo^ diagInfo);
	SIResult SIScanner::SI_GetEvent(SIEventInfo^ eventInfo);

private:
	void NativeValueToManaged(uint16 nativeType, SINative::SIValue* nativeVal, ScannerMng::SIValue^% mngVal);
	void NativeValueToManagedArray(int32 numItems, uint16 nativeType, SINative::SIValue* nativeVal, ScannerMng::SIValue^% mngVal);
	void ManagedValueToNative(uint16 nativeType, ScannerMng::SIValue^% mngVal, SINative::SIValue* nativeVal);
	void ManagedValueToNativeArray(int32 numItems, uint16 nativeType, ScannerMng::SIValue^% mngVal, SINative::SIValue* nativeVal);

private:
	Scanner				m_scanner;
	IntPtr				m_pMem;

	// An array of IntPtrs that may get allocated in SI_SetProperty() if the property type
	// happens to be an array of strings or a list of strings. (There are currently no properties
	// with that type.)
	cli::array<IntPtr>^	m_pIntPtrMem;

public:
	ProgressCallbackDelegate^ m_progressCallback;
	static SIScanner^ s_callbackInstance;

};

SIScanner::SIScanner(String^ dllFilename)
	:m_pMem(IntPtr::Zero)
{
	// convert the string to ANSI
	IntPtr ip = Marshal::StringToHGlobalAnsi(dllFilename);
	const char* str = static_cast<const char*>(ip.ToPointer());

	bool result = m_scanner.Attach(str);
	Marshal::FreeHGlobal( ip );

	if(!result)
	{
		String^ tmp = gcnew String("Cannot load the file: ");
		tmp += dllFilename;
		throw gcnew ApplicationException(tmp);
	}

}

SIResult SIScanner::SI_OpenInterface(String^ scannerName)
{

	IntPtr ip = Marshal::StringToHGlobalAnsi(scannerName);
	const char* str = static_cast<const char*>(ip.ToPointer());

	SIResult res = (SIResult)m_scanner.SI_OpenInterface(str);

	Marshal::FreeHGlobal( ip );

	return res;
}

SIResult SIScanner::SI_CloseInterface()
{
	return (SIResult)m_scanner.SI_CloseInterface();
}

SIResult SIScanner::SI_IsCalibrated(SICalibrationState% pState)
{
	SINative::SICalibrationState calState;
	SIResult result = (SIResult)m_scanner.SI_IsCalibrated(&calState);

	if(result == SIResult::SIR_SUCCESS)
	{
		pState = (calState == SINative::SI_TRUE) ? 
			SICalibrationState::CALIBRATED : 
			SICalibrationState::NOT_CALIBRATED;
	}
	return result;
}


static int32 ProgressCallback(uint32 percentComplete)
{
	int32 result = 0;

	if(SIScanner::s_callbackInstance)
	{
		result = (int32)SIScanner::s_callbackInstance->m_progressCallback(percentComplete);
	}	
	
	return result;
}


SIResult SIScanner::SI_Calibrate(SICalibrationTarget calTarget, ProgressCallbackDelegate^ progressCallback)
{
	uint32 ct = (uint32)calTarget;
	SIResult result;

	if(s_callbackInstance != nullptr)
	{
		return SIResult::SIR_SCANNER_BUSY;
	}

	if(progressCallback != nullptr)
	{
		m_progressCallback = progressCallback;
		s_callbackInstance = this;
	
		result = (SIResult)m_scanner.SI_Calibrate(ct, ProgressCallback);

		m_progressCallback = nullptr;
		s_callbackInstance = nullptr;
	}
	else
	{
		result = (SIResult)m_scanner.SI_Calibrate(ct, NULL);
	}

	return result;
}

SIResult SIScanner::SI_Clean(ProgressCallbackDelegate^ progressCallback)
{
	SIResult result;

	if(s_callbackInstance != nullptr)
	{
		return SIResult::SIR_SCANNER_BUSY;
	}

	if(progressCallback != nullptr)
	{
		m_progressCallback = progressCallback;
		s_callbackInstance = this;
	
		result = (SIResult)m_scanner.SI_Clean(ProgressCallback);

		m_progressCallback = nullptr;
		s_callbackInstance = nullptr;
	}
	else
	{
		result = (SIResult)m_scanner.SI_Clean(NULL);
	}

	return result;
}

SIResult SIScanner::SI_GetPaperStatus(SIPaperStatus% paperStatus)
{
	SINative::SIPaperStatus ps;
	SIResult result = (SIResult)m_scanner.SI_GetPaperStatus(&ps);

	if(result == SIResult::SIR_SUCCESS)
	{
		paperStatus = (SIPaperStatus)ps;
	}
	return result;
}

SIResult SIScanner::SI_GetButtonStatus(UInt32 buttonNumber, SIButtonStatus% buttonStatus)
{
	SINative::SIButtonStatus bs;
	SIResult result = (SIResult)m_scanner.SI_GetButtonStatus(buttonNumber, &bs);

	if(result == SIResult::SIR_SUCCESS)
	{
		buttonStatus = (SIButtonStatus)bs;
	}
	return result;
}

SIResult SIScanner::SI_GetScannerStatus(SIScannerStatus% scannerStatus)
{
	SINative::SIScannerStatus ss;
	SIResult result = (SIResult)m_scanner.SI_GetScannerStatus(&ss);

	if(result == SIResult::SIR_SUCCESS)
	{
		scannerStatus = (SIScannerStatus)ss;
	}
	return result;
}

SIResult SIScanner::SI_StartScan()
{
	SIResult result = (SIResult)m_scanner.SI_StartScan();

	return result;
}

SIResult SIScanner::SI_EndScan()
{
	return (SIResult)m_scanner.SI_EndScan();
}

SIResult SIScanner::SI_Feed()
{
	return (SIResult)m_scanner.SI_Feed();
}

SIResult SIScanner::SI_FeedPaperOut()
{
	return (SIResult)m_scanner.SI_FeedPaperOut();
}


SIResult SIScanner::SI_ReadImageData(cli::array<Byte>^ buffer, UInt32 numberOfLinesToRead, UInt32 pageNumber, UInt32% numberOfLinesReturned )
{
	SIResult result;

	uint32 numLinesReturnedLocal = 0;
	pin_ptr<Byte> pBuffer = &buffer[0];

	result = (SIResult)m_scanner.SI_ReadImageData(pBuffer, numberOfLinesToRead, pageNumber, &numLinesReturnedLocal);
	
	if(result == SIResult::SIR_SUCCESS)
	{
		numberOfLinesReturned = numLinesReturnedLocal;
	}

	return result;

}

void SIScanner::NativeValueToManaged(uint16 nativeType, SINative::SIValue* nativeVal, ScannerMng::SIValue^% mngVal)
{
	switch(nativeType)
	{
		case SINative::SI_BOOL:
			mngVal->bVal = nativeVal->bVal;		
			break;
		case SINative::SI_INT32:
			mngVal->iVal = nativeVal->iVal;		
			break;
		case SINative::SI_FLOAT32:
			mngVal->fVal = nativeVal->fVal;		
			break;
		case SINative::SI_STR:
			mngVal->strVal = gcnew String(nativeVal->strVal);
			break;

	};

	return;
}

// For the case of arrays, marshal native arrays to managed arrays. 
// Even if the number of items is zero, this function will at least allocate 
// a zero-length managed array.
void SIScanner::NativeValueToManagedArray(int32 numItems, uint16 nativeType, SINative::SIValue* nativeVal, ScannerMng::SIValue^% mngVal)
{
		switch(nativeType)
		{
			case SINative::SI_INT32:
				{
					mngVal->piVal = gcnew cli::array<System::Int32>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::Int32> pInt = &(mngVal->piVal[0]);
						memcpy(pInt, nativeVal->piVal, numItems * sizeof(int32));
					}
				}
				break;
			case SINative::SI_UINT32:
				{
					mngVal->puiVal = gcnew cli::array<System::UInt32>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::UInt32> pUInt = &(mngVal->puiVal[0]);
						memcpy(pUInt, nativeVal->puiVal, numItems * sizeof(uint32));
					}
				}
				break;

			case SINative::SI_INT16:
				{
					mngVal->psVal = gcnew cli::array<System::Int16>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::Int16> pInt = &(mngVal->psVal[0]);
						memcpy(pInt, nativeVal->psVal, numItems * sizeof(int16));
					}
				}
				break;
			case SINative::SI_UINT16:
				{
					mngVal->pusVal = gcnew cli::array<System::UInt16>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::UInt16> pUInt = &(mngVal->pusVal[0]);
						memcpy(pUInt, nativeVal->pusVal, numItems * sizeof(uint16));
					}
				}
				break;
			case SINative::SI_INT8:
				{
					mngVal->pcVal = gcnew cli::array<System::SByte>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::SByte> pSByte = &(mngVal->pcVal[0]);
						memcpy(pSByte, nativeVal->pcVal, numItems * sizeof(int8));
					}
				}
				break;
			case SINative::SI_UINT8:
				{
					mngVal->pucVal = gcnew cli::array<System::Byte>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::Byte> pByte = &(mngVal->pucVal[0]);
						memcpy(pByte, nativeVal->pucVal, numItems * sizeof(uint8));
					}
				}
				break;
			case SINative::SI_FLOAT32:
				{
					mngVal->pfVal = gcnew cli::array<System::Single>(numItems);
					if(numItems > 0)
					{
						pin_ptr<System::Single> pSingle = &(mngVal->pfVal[0]);
						memcpy(pSingle, nativeVal->pfVal, numItems * sizeof(float32));
					}
				}
				break;

			case SINative::SI_BOOL:
				{
					mngVal->pbVal = gcnew cli::array<System::Boolean>(numItems);
				
					// Since System::Boolean is not the same size as SI_BOOL (int32), we'll need to copy the slow way.
					// (Currently, no scanner property uses an array of SI_BOOLS. This is only for possible future use).
					for(int i=0; i < numItems; i++)
					{
						mngVal->pbVal[i] = (nativeVal->pbVal[i] == SINative::SI_TRUE) ? true : false;
					}
				}
				break;

			case SINative::SI_STR:
				{
					mngVal->pstrVal = gcnew cli::array<System::String^>(numItems);
					if(numItems > 0)
					{
						for(int i=0; i < numItems; i++)
						{
							mngVal->pstrVal[i] = gcnew System::String(nativeVal->pstrVal[i]);
						}

					}
				}
				break;

		};


	return;
}

SIResult SIScanner::SI_GetProperty(SIProperty^ prop)
{

	if(prop == nullptr)
	{
		return SIResult::SIR_BAD_PARAMETER;
	}

	SINative::SIProperty nativeProp;
	nativeProp.propertyID = prop->propertyID;

	SIResult result = (SIResult)m_scanner.SI_GetProperty(&nativeProp);

	if(result != SIResult::SIR_SUCCESS)
	{
		return result;
	}

	// Marshal
	prop->containerType = nativeProp.containerType;
	prop->itemType = nativeProp.itemType;
	prop->access = nativeProp.access;

	switch(nativeProp.containerType)
	{
	case SINative::SICON_SINGLE:

		if((prop->single == nullptr) || (! prop->single->GetType()->Equals(ScannerMng::SISingle::typeid)))
		{
			prop->single = gcnew SISingle();
		}
		NativeValueToManaged(nativeProp.itemType, &nativeProp.single.current, prop->single->current);
		NativeValueToManaged(nativeProp.itemType, &nativeProp.single.def, prop->single->def);
		break;

	case SINative::SICON_RANGE:
		if((prop->range == nullptr) || (! prop->range->GetType()->Equals(ScannerMng::SIRange::typeid)))
		{
			prop->range = gcnew SIRange();
		}
		NativeValueToManaged(nativeProp.itemType, &nativeProp.range.current, prop->range->current);
		NativeValueToManaged(nativeProp.itemType, &nativeProp.range.def, prop->range->def);
		NativeValueToManaged(nativeProp.itemType, &nativeProp.range.minimum, prop->range->minimum);
		NativeValueToManaged(nativeProp.itemType, &nativeProp.range.maximum, prop->range->maximum);
		NativeValueToManaged(nativeProp.itemType, &nativeProp.range.stepSize, prop->range->stepSize);
		break;

	case SINative::SICON_LIST:
		if((prop->list == nullptr) || (! prop->list->GetType()->Equals(ScannerMng::SIList::typeid)))
		{
			prop->list = gcnew SIList();
		}

		prop->list->numItems = nativeProp.list.numItems;
		NativeValueToManaged(nativeProp.itemType, &nativeProp.list.current, prop->list->current);
		NativeValueToManaged(nativeProp.itemType, &nativeProp.list.def, prop->list->def);
		NativeValueToManagedArray(nativeProp.list.numItems, 
			nativeProp.itemType, &nativeProp.list.items, prop->list->items);

		break;

	case SINative::SICON_ARRAY:
		if((prop->array == nullptr) || (! prop->array->GetType()->Equals(ScannerMng::SIArray::typeid)))
		{
			prop->array = gcnew SIArray();
		}

		prop->array->numItems = nativeProp.array.numItems;

		NativeValueToManagedArray(nativeProp.array.numItems,
			nativeProp.itemType, &nativeProp.array.items, prop->array->items);

		break;

	default:
		break;

	};

	return result;
}

void SIScanner::ManagedValueToNative(uint16 nativeType, ScannerMng::SIValue^% mngVal, SINative::SIValue* nativeVal)
{
	switch(nativeType)
	{
		case SINative::SI_BOOL:
			nativeVal->bVal = mngVal->bVal;		
			break;
		case SINative::SI_INT32:
			nativeVal->iVal = mngVal->iVal;		
			break;
		case SINative::SI_FLOAT32:
			nativeVal->fVal = mngVal->fVal;		
			break;
		case SINative::SI_STR:
			if(mngVal->strVal != nullptr)
			{
				m_pMem = Marshal::StringToHGlobalAnsi(mngVal->strVal);
				const char* str = static_cast<const char*>(m_pMem.ToPointer());
				nativeVal->strVal = str;
				
				// Note that the memory pointed to by m_pMem will be
				// freed elsewhere before returning to the calling program.
			}
			break;

	};

	return;
}

void SIScanner::ManagedValueToNativeArray(int32 numItems, uint16 nativeType, ScannerMng::SIValue^% mngVal, SINative::SIValue* nativeVal)
{
		switch(nativeType)
		{
			case SINative::SI_INT32:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->piVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(int32));
					nativeVal->piVal = (int32*)m_pMem.ToPointer();
					pin_ptr<System::Int32> pInt = &(mngVal->piVal[0]);
					memcpy(nativeVal->piVal, pInt,  numItems * sizeof(int32));
				}
				break;
			case SINative::SI_UINT32:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->puiVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(uint32));
					nativeVal->puiVal = (uint32*)m_pMem.ToPointer();
					pin_ptr<System::UInt32> pUInt = &(mngVal->puiVal[0]);
					memcpy(nativeVal->puiVal, pUInt,  numItems * sizeof(uint32));
				}
				break;
			case SINative::SI_INT16:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->psVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(int16));
					nativeVal->psVal = (int16*)m_pMem.ToPointer();
					pin_ptr<System::Int16> pInt = &(mngVal->psVal[0]);
					memcpy(nativeVal->psVal, pInt,  numItems * sizeof(int16));
				}
				break;
			case SINative::SI_UINT16:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->pusVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(uint16));
					nativeVal->pusVal = (uint16*)m_pMem.ToPointer();
					pin_ptr<System::UInt16> pUInt = &(mngVal->pusVal[0]);
					memcpy(nativeVal->pusVal, pUInt,  numItems * sizeof(uint16));
				}
				break;

			case SINative::SI_INT8:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->pcVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(int8));
					nativeVal->pcVal = (int8*)m_pMem.ToPointer();
					pin_ptr<System::SByte> pSByte = &(mngVal->pcVal[0]);
					memcpy(nativeVal->pcVal, pSByte,  numItems * sizeof(int8));
				}
				break;
			case SINative::SI_UINT8:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->pucVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(uint8));
					nativeVal->pucVal = (uint8*)m_pMem.ToPointer();
					pin_ptr<System::Byte> pByte = &(mngVal->pucVal[0]);
					memcpy(nativeVal->pucVal, pByte,  numItems * sizeof(uint8));
				}
				break;
			case SINative::SI_FLOAT32:
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->pfVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(float32));
					nativeVal->pfVal = (float32*)m_pMem.ToPointer();
					pin_ptr<System::Single> pSingle = &(mngVal->pfVal[0]);
					memcpy(nativeVal->pfVal, pSingle,  numItems * sizeof(float32));
				}
				break;
			case SINative::SI_BOOL:

				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->pbVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(int32));
					nativeVal->pbVal = (int32*)m_pMem.ToPointer();
					pin_ptr<System::Boolean> pInt = &(mngVal->pbVal[0]);
				
					// Since System::Boolean is not the same size as SI_BOOL (int32), we'll need to copy the slow way.
					// (Currently, no scanner property uses an array of SI_BOOLS. This is only for possible future use).
					for(int i=0; i < numItems; i++)
					{
						nativeVal->pbVal[i] = (mngVal->pbVal[i]) ? SINative::SI_TRUE : SINative::SI_FALSE;
					}
				}
				break;

			case SINative::SI_STR:
				// This is the most difficult case. Actually, there aren't yet any properties 
				// that are an array of strings or a list of strings.
				// We need to allocate space for each individual native string and then copy 
				// the managed string there. Then after calling the native SI_SetProperty() 
				// we need to deallocate that memory.
				if(numItems > 0)
				{
					numItems = min(numItems, mngVal->pstrVal->Length);

					m_pMem = Marshal::AllocHGlobal(numItems * sizeof(char *));
					nativeVal->pstrVal = static_cast<char**>(m_pMem.ToPointer());

					// Create an array to hold pointers to the memory allocated for each unmanaged string.
					// These will be freed before returning to the calling program.
					m_pIntPtrMem = gcnew cli::array<IntPtr>(numItems);

					for(int i = 0; i < numItems; i++)
					{
						m_pIntPtrMem[i] = Marshal::StringToHGlobalAnsi(mngVal->pstrVal[i]);
						nativeVal->pstrVal[i] = static_cast<char*>(m_pIntPtrMem[i].ToPointer());
					}

				}
				break;

			default:

				break;
		};

	return;
}




SIResult SIScanner::SI_SetProperty(SIProperty^ prop)
{
	if(prop == nullptr)
	{
		return SIResult::SIR_BAD_PARAMETER;
	}

	SINative::SIProperty nativeProp;
	SIResult result;

	nativeProp.propertyID = prop->propertyID;
	nativeProp.containerType = prop->containerType;
	nativeProp.itemType = prop->itemType;

	if(IntPtr::Zero != m_pMem)
	{
		Marshal::FreeHGlobal(m_pMem);
		m_pMem = IntPtr::Zero;
	}

	if(m_pIntPtrMem != nullptr)
	{
		for(int i = 0; i < m_pIntPtrMem->Length; i++)
		{
			Marshal::FreeHGlobal(m_pIntPtrMem[i]);
		}
		m_pIntPtrMem = nullptr;
	}

	switch(nativeProp.containerType)
	{
		case SINative::SICON_SINGLE:
			if(	(prop->single == nullptr) || 
				(prop->single->current == nullptr) ||
				(prop->single->def == nullptr))
			{
				return SIResult::SIR_BAD_PARAMETER;
			}
			ManagedValueToNative(nativeProp.itemType, prop->single->current, &nativeProp.single.current);
			ManagedValueToNative(nativeProp.itemType, prop->single->def, &nativeProp.single.def);

			break;

		case SINative::SICON_RANGE:
			if(	(prop->range == nullptr) || 
				(prop->range->current == nullptr) ||
				(prop->range->def == nullptr) ||
				(prop->range->stepSize == nullptr) ||
				(prop->range->minimum == nullptr) ||
				(prop->range->maximum == nullptr))
			{
				return SIResult::SIR_BAD_PARAMETER;
			}

			ManagedValueToNative(nativeProp.itemType, prop->range->current, &nativeProp.range.current);
			ManagedValueToNative(nativeProp.itemType, prop->range->def, &nativeProp.range.def);
			ManagedValueToNative(nativeProp.itemType, prop->range->stepSize, &nativeProp.range.stepSize);
			ManagedValueToNative(nativeProp.itemType, prop->range->minimum, &nativeProp.range.minimum);
			ManagedValueToNative(nativeProp.itemType, prop->range->maximum, &nativeProp.range.maximum);

			break;

		case SINative::SICON_LIST:
			if(	(prop->list == nullptr) || 
				(prop->list->current == nullptr) ||
				(prop->list->def == nullptr) ||
				(prop->list->items == nullptr))
			{
				return SIResult::SIR_BAD_PARAMETER;
			}

			nativeProp.list.numItems = prop->list->numItems;
			ManagedValueToNative(nativeProp.itemType, prop->list->current, &nativeProp.list.current);
			ManagedValueToNative(nativeProp.itemType, prop->list->def, &nativeProp.list.def);
			ManagedValueToNativeArray(nativeProp.list.numItems, 
				nativeProp.itemType, prop->list->items, &nativeProp.list.items);

			break;

		case SINative::SICON_ARRAY:
			if(	(prop->array == nullptr) || 
				(prop->array->items == nullptr))
			{
				return SIResult::SIR_BAD_PARAMETER;
			}

			nativeProp.array.numItems = prop->array->numItems;
			ManagedValueToNativeArray(nativeProp.array.numItems,
				nativeProp.itemType, prop->array->items, &nativeProp.array.items);

			break;

		default:
			break;
	};

	result = (SIResult)m_scanner.SI_SetProperty(&nativeProp);

	// If the property type happens to be an array of strings or a list of strings, then
	// it will have allocated an array of IntPtrs which need to be freed.
	// with that type.)
	if(m_pIntPtrMem != nullptr)
	{
		for(int i = 0; i < m_pIntPtrMem->Length; i++)
		{
			Marshal::FreeHGlobal(m_pIntPtrMem[i]);
		}
		m_pIntPtrMem = nullptr;
	}

	if(IntPtr::Zero != m_pMem)
	{
		Marshal::FreeHGlobal(m_pMem);
		m_pMem = IntPtr::Zero;
	}


	return result;
}


SIResult SIScanner::SI_Reset()
{
	return (SIResult)m_scanner.SI_Reset();
}

SIResult SIScanner::SI_GetLastError()
{
	return (SIResult)m_scanner.SI_GetLastError();
}

SIResult SIScanner::SI_GetLastErrorText(String^% errorText)
{
	SIResult result;
	char *pErrText = NULL;

	result = (SIResult)m_scanner.SI_GetLastErrorText(&pErrText);

	if(result == SIResult::SIR_SUCCESS)
	{
		errorText = gcnew String(pErrText);
	}
	return result;
}

SIResult SIScanner::SI_Diagnostic(SIDiagTest test, SIDiagInfo^ diagInfo)
{
	SIResult result = SIResult::SIR_BAD_PARAMETER;

	if(test == SIDiagTest::SI_DIAG_HW_PRESENCE)
	{
		SINative::SIDiagHwPresenceInfo localInfo;
		result = (SIResult)m_scanner.SI_Diagnostic(SINative::SI_DIAG_HW_PRESENCE, &localInfo);

		if(result == SIResult::SIR_SUCCESS)
		{
			((SIDiagHwPresenceInfo^) diagInfo)->result = localInfo.result;
		}
	}
	else if( test == SIDiagTest::SI_DIAG_READ_PERFORMANCE)
	{
		SINative::SIDiagReadPerfInfo localInfo;
		localInfo.readLengthInKB = ((SIDiagReadPerfInfo^) diagInfo)->readLengthInKB;
		result = (SIResult)m_scanner.SI_Diagnostic(SINative::SI_DIAG_READ_PERFORMANCE, &localInfo);

		if(result == SIResult::SIR_SUCCESS)
		{
			((SIDiagReadPerfInfo^) diagInfo)->timeInMicroSec = localInfo.timeInMicroSec;
		}
	}


	return result;

}

SIResult SIScanner::SI_GetEvent(SIEventInfo^ eventInfo)
{
	SIResult result = SIResult::SIR_SUCCESS;

	if(eventInfo->eventDataSize == 0)
	{
		return result;
	}
	if(eventInfo->eventDataSize > MAX_EVENT_DATA_SIZE)
	{
		return SIResult::SIR_BAD_PARAMETER;
	}

	SINative::SIEventInfo localInfo;

	localInfo.eventDataSize = eventInfo->eventDataSize;
	for(int i = 0; i < localInfo.eventDataSize; i++)
	{
		localInfo.eventData[i] = eventInfo->eventData[i];
	}

	result = (SIResult)m_scanner.SI_GetEvent(&localInfo);

	if(result == SIResult::SIR_SUCCESS)
	{
		eventInfo->eventFlags = localInfo.eventFlags;
	}

	return result;

}



}	// end of namespace ScannerMng

