// CalibrationDialog.cpp : implementation file
//

#include "stdafx.h"
#include "SpeedbirdTest.h"
#include "CalibrationDialog.h"
#include "Scanner.h"
#include ".\calibrationdialog.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC(CalibrationDialog, DialogBarEx)
CalibrationDialog::CalibrationDialog(CWnd* pParent /*=NULL*/)
{
}

CalibrationDialog::~CalibrationDialog()
{
}

void CalibrationDialog::DoDataExchange(CDataExchange* pDX)
{
	DialogBarEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CALIBRATION_STATE_STATIC, m_calibrationStateStatic);
}


BEGIN_MESSAGE_MAP(CalibrationDialog, DialogBarEx)
	ON_BN_CLICKED(IDC_GET_CALIBRATION_STATE_BUTTON, OnBnClickedGetCalibrationStateButton)
	ON_BN_CLICKED(IDC_DO_CALIBRATION_BUTTON, OnBnClickedDoCalibrationButton)
	ON_BN_CLICKED(IDC_CLEAN_BUTTON, OnBnClickedCleanButton)
END_MESSAGE_MAP()



// CalibrationDialog message handlers

BOOL CalibrationDialog::OnInitDialogBar()
{
	DialogBarEx::OnInitDialogBar();

	m_calibrationStateStatic.SetWindowText("Unknown");

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}


void CalibrationDialog::OnBnClickedGetCalibrationStateButton()
{
	// TODO: Add your control notification handler code here

	SIResult result;
	SICalibrationState state = SI_FALSE;

	result = g_scanner.SI_IsCalibrated(&state);

	if(result == SIR_SUCCESS)
	{		
		if(state == SI_TRUE)
		{
			m_calibrationStateStatic.SetWindowText("Calibrated");
		}
		else if (state == SI_FALSE)
		{
			m_calibrationStateStatic.SetWindowText("Not Calibrated");
		}
		else
		{
			m_calibrationStateStatic.SetWindowText("Unknown");
			MessageBox("Output parameter set to invalid value.");
		}
	}
	else
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}

}

void CalibrationDialog::OnBnClickedDoCalibrationButton()
{
	CProgressDialog dlg(CProgressDialog::OPER_CALIBRATE);

	// Run the CProgressDialog dialog which handles the calibration.
	dlg.DoModal();
}

void CalibrationDialog::OnBnClickedCleanButton()
{

	CProgressDialog dlg(CProgressDialog::OPER_CLEAN);

	// Run the CProgressDialog dialog which handles the cleaning process.
	dlg.DoModal();

#if 0
	SIResult result;
	result = g_scanner.SI_Clean(NULL);
	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}
#endif
}



// CProgressDialog dialog

IMPLEMENT_DYNAMIC(CProgressDialog, CDialog)
CProgressDialog::CProgressDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CProgressDialog::IDD, pParent)
{
}

CProgressDialog::CProgressDialog(Operation operation, CWnd* pParent /*=NULL*/)
	: CDialog(CProgressDialog::IDD, pParent)
{
	m_operation = operation;
}



CProgressDialog::~CProgressDialog()
{
}

void CProgressDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_PROGRESS, m_progressControl);
	DDX_Control(pDX, IDC_PROG_STATIC, m_progressStatic);
}


BEGIN_MESSAGE_MAP(CProgressDialog, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(ID_CANCEL_OPERATION, OnBnClickedCancelOperation)
END_MESSAGE_MAP()

CProgressDialog*	CProgressDialog::pToInstance = NULL;
bool				CProgressDialog::cancelOperation = false;


// CProgressDialog message handlers

BOOL CProgressDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// The callback function is static, so we need to save this instance 
	// to enable the callback to access it.
	pToInstance = this;
	cancelOperation = false;

	if(m_operation == OPER_CALIBRATE)
	{
		m_progressStatic.SetWindowText("Calibrating...");
	}
	else
	{
		m_progressStatic.SetWindowText("Cleaning...");
	}

	// Allow 100 ms time for the dialog to initialize before starting 
	// the operation.
	SetTimer(0, 100, 0);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}


int32 SICALLBACK CProgressDialog::ProgressCallback(uint32 percentCompleted)
{
	
	// Update the progress control with the current progress.
	// By default, the range of a progress control is already 0 to 100.
	pToInstance->m_progressControl.SetPos(percentCompleted);
    
	MSG msg;
	 
	// Process Windows messages so that the cancel button will work and the 
	// current progress will be displayed.
	while( PeekMessage(&msg, pToInstance->GetSafeHwnd(),  0, 0, PM_REMOVE))
    { 
		TranslateMessage(&msg); 
		DispatchMessage(&msg); 
        
    } 

	// Return SI_FALSE if the user hit the cancel button.
	return cancelOperation ? SI_FALSE : SI_TRUE;
}


void CProgressDialog::OnTimer(UINT_PTR nIDEvent)
{
	// Stop the timer. We only need it to fire once.
	KillTimer(0);

	// Start the calibration process. This function will not return
	// until calibration is done or has been canceled.
	SIResult result;

	if(m_operation == OPER_CALIBRATE)
	{
		result = g_scanner.SI_Calibrate(SI_CT_DEFAULT, ProgressCallback);
	}
	else
	{
		result = g_scanner.SI_Clean(ProgressCallback);
	}

	if(result != SIR_SUCCESS)
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}

	// The operation is done. So close this dialog.
	PostMessage(WM_COMMAND, IDCANCEL, 0);

}

void CProgressDialog::OnBnClickedCancelOperation()
{
	// Set the cancelOperation variable that the callback function
	// will see and cancel the operation next time it gets called.
	cancelOperation = true;

	m_progressStatic.SetWindowText("Cancelling...");

}
