// OpenCloseDialog.cpp : implementation file
//

#include "stdafx.h"
#include "SpeedbirdTest.h"
#include "OpenCloseDialog.h"
#include "MainFrm.h"
#include "ScannerAPI.h"
#include "Scanner.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


typedef struct
{
	char name[40];				// The name used to open the interface with.
	char DLLFilename[20];		
} ScannerInfo;

//////////////  Define all scanners here. //////////////////////////////
//
//
////////////////////////////////////////////////////////////////////////
ScannerInfo g_scannerList[] = 
{	
//	Scanner name as used		DLL filename for the 
//	in SI_Openinterface			scanner
//	---------------------		-----------------
	{"DocketPORT487",			"DPORT487.DLL"},
	{"DocketPORT488",			"DPORT488.DLL"},
	{"DocketPORT467",			"DPORT467.DLL"},
	{"DocketPORT468",			"DPORT468.DLL"},
	{"DocketPORT667",			"DPORT667.DLL"},
	{"DocketPORT687",			"DPORT687.DLL"},
};


// OpenCloseDialog dialog

IMPLEMENT_DYNAMIC(OpenCloseDialog, DialogBarEx)
OpenCloseDialog::OpenCloseDialog(CWnd* pParent /*=NULL*/)
	: m_interfaceIsOpen(false)
{
}

OpenCloseDialog::~OpenCloseDialog()
{
}

void OpenCloseDialog::DoDataExchange(CDataExchange* pDX)
{
	DialogBarEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SCANNER_NAME_COMBO, m_scannerName);
	DDX_Control(pDX, IDC_INTERFACE_STATE_STATIC, m_interfaceState);
}


BEGIN_MESSAGE_MAP(OpenCloseDialog, DialogBarEx)
	ON_BN_CLICKED(IDC_OPEN_INTERFACE_BUTTON, OnBnClickedOpenInterfaceButton)
	ON_BN_CLICKED(IDC_CLOSE_INTERFACE_BUTTON, OnBnClickedCloseInterfaceButton)
//	ON_WM_CREATE()
ON_CBN_SELENDOK(IDC_SCANNER_NAME_COMBO, OnSelectionMadeInScannerNameCombo)
END_MESSAGE_MAP()


// OpenCloseDialog message handlers




void OpenCloseDialog::OnBnClickedOpenInterfaceButton()
{
	// TODO: Add your control notification handler code here

	// Retrieve scanner name from the combo box
	int currentScannerIndex = m_scannerName.GetCurSel();

	CString scannerName;
	scannerName = g_scannerList[currentScannerIndex].name;

	SIResult result;
	result = g_scanner.SI_OpenInterface(scannerName);

	if(result == SIR_SUCCESS)
	{
		m_interfaceIsOpen = true;
		m_interfaceState.SetWindowText("Interface state: Open");

		// Signal the parent that the interface has been opened.
		((CMainFrame *)GetParent())->InterfaceOpened();
	}
	else
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);

		MessageBox(pErrorText);
	}

}

void OpenCloseDialog::OnBnClickedCloseInterfaceButton()
{
	// TODO: Add your control notification handler code here

	SIResult result;

	result = g_scanner.SI_CloseInterface();

	if(result == SIR_SUCCESS)
	{
		m_interfaceIsOpen = false;
		m_interfaceState.SetWindowText("Interface state: Closed");

	}
	else
	{
		char *pErrorText;
		g_scanner.SI_GetLastErrorText(&pErrorText);
		MessageBox(pErrorText);
	}

}



BOOL OpenCloseDialog::OnInitDialogBar()
{
	DialogBarEx::OnInitDialogBar();

	// disable buttons
	GetDlgItem(IDC_OPEN_INTERFACE_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_CLOSE_INTERFACE_BUTTON)->EnableWindow(FALSE);

	// Disable all buttons on the dialog bar except the Open/Close button.
	((CMainFrame*)AfxGetMainWnd())->EnableNonOpenCloseButtons(FALSE);

	m_scannerName.SetCurSel(0);
	m_interfaceState.SetWindowText("Interface state: Closed");


	// Add scanner names to the listbox
	int numScanners = sizeof(g_scannerList) / sizeof(ScannerInfo);
	for(int i = 0; i < numScanners; i++)
	{
		m_scannerName.InsertString(i, g_scannerList[i].name);
	}
	m_scannerName.SetWindowText("Choose Scanner");


	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}


void OpenCloseDialog::OnSelectionMadeInScannerNameCombo()
{
	// If the DLL is currently loaded, close the scanner interface and
	// unload the DLL.
	if(g_scanner.IsAttached())
	{

		g_scanner.SI_CloseInterface();
		m_interfaceState.SetWindowText("Interface state: Closed");

		g_scanner.Detach();
	}


	int currentScannerIndex = m_scannerName.GetCurSel();


	// Use the DLL filename from the scanner list for the selected scanner.
	bool isLoaded;

	isLoaded = g_scanner.Attach(g_scannerList[currentScannerIndex].DLLFilename);

	if(isLoaded)
	{
		// Enable Open and Close Interface buttons
		GetDlgItem(IDC_OPEN_INTERFACE_BUTTON)->EnableWindow(TRUE);
		GetDlgItem(IDC_CLOSE_INTERFACE_BUTTON)->EnableWindow(TRUE);

		// Enable all buttons on the dialog bar except the Open/Close button.
		((CMainFrame*)AfxGetMainWnd())->EnableNonOpenCloseButtons(TRUE);

	}
	else
	{
		// Disable Open and Close Interface buttons
		GetDlgItem(IDC_OPEN_INTERFACE_BUTTON)->EnableWindow(FALSE);
		GetDlgItem(IDC_CLOSE_INTERFACE_BUTTON)->EnableWindow(FALSE);

		// Disable all buttons on the dialog bar except the Open/Close button.
		((CMainFrame*)AfxGetMainWnd())->EnableNonOpenCloseButtons(FALSE);

		CString str = "Could not find the file: ";
		str += g_scannerList[currentScannerIndex].DLLFilename;
		MessageBox( str );
		return;

	}

}
