// ScanParametersDialog.cpp : implementation file
//

#include "stdafx.h"
#include "SpeedbirdTest.h"
#include "ScanParametersDialog.h"
#include "assert.h"
#include "math.h"
#include "Scanner.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// ScanParametersDialog dialog

IMPLEMENT_DYNAMIC(ScanParametersDialog, DialogBarEx)
ScanParametersDialog::ScanParametersDialog(CWnd* pParent /*=NULL*/)
	:m_suppressApplyButtonUpdate(false)
{
}

ScanParametersDialog::~ScanParametersDialog()
{
}

void ScanParametersDialog::DoDataExchange(CDataExchange* pDX)
{
	DialogBarEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_MODE_COMBO, m_modeCombo);
	DDX_Control(pDX, IDC_RESOLUTION_COMBO, m_resolutionCombo);
	DDX_Control(pDX, IDC_LEFT_OFFSET_EDIT, m_leftOffsetEdit);
	DDX_Control(pDX, IDC_TOP_OFFSET_EDIT, m_topOffsetEdit);
	DDX_Control(pDX, IDC_WIDTH_EDIT, m_widthEdit);
	DDX_Control(pDX, IDC_LENGTH_EDIT, m_lengthEdit);
	DDX_Control(pDX, IDC_CHANNEL_ORDER_COMBO, m_channelOrderCombo);
	DDX_Control(pDX, IDC_BRIGHTNESS_EDIT, m_brightnessEdit);
	DDX_Control(pDX, IDC_CONTRAST_EDIT, m_contrastEdit);
	DDX_Control(pDX, IDC_THRESHOLD_EDIT, m_thresholdEdit);
	DDX_Control(pDX, IDC_GAMMA_EDIT, m_gammaEdit);
	DDX_Control(pDX, IDC_HIGHLIGHT_EDIT, m_highlightEdit);
	DDX_Control(pDX, IDC_SHADOW_EDIT, m_shadowEdit);
	DDX_Control(pDX, IDC_DESCREEN_CHECKBOX, m_descreenCheckbox);
	DDX_Control(pDX, IDC_DUPLEX_CHECKBOX, m_duplexCheckbox);
	DDX_Control(pDX, IDC_EOP_DETECT_CHECKBOX, m_EOPDetectCheckbox);
	DDX_Control(pDX, IDC_SPOOLER_CHECKBOX, m_spoolerCheckbox);
	DDX_Control(pDX, IDC_PLANAR_CHUNKY_COMBO, m_planarChunkyCombo);
	DDX_Control(pDX, IDC_PHOTO_INTERP_COMBO2, m_photoInterpCombo);
	DDX_Control(pDX, IDC_FEED_DIRECTION_COMBO, m_feedDirectionCombo);
	DDX_Control(pDX, IDC_SET_SCAN_RATE_CHECKBOX, m_setScanRateCheckbox);
	DDX_Control(pDX, IDC_SCAN_RATE_EDIT, m_scanRateEdit);
	DDX_Control(pDX, IDC_FEED_RATE_EDIT, m_feedRateEdit);
	DDX_Control(pDX, IDC_GRAYSCALE_LUT_COMBO, m_grayLUTCombo);
	DDX_Control(pDX, IDC_RED_LUT_COMBO, m_redLUTCombo);
	DDX_Control(pDX, IDC_GREEN_LUT_COMBO, m_greenLUTCombo);
	DDX_Control(pDX, IDC_BLUE_LUT_COMBO, m_blueLUTCombo);
	DDX_Control(pDX, IDC_LINE_WIDTH_IN_BYTES_STATIC, m_lineWidthInBytesStatic);
	DDX_Control(pDX, IDC_BITS_PER_PIXEL_STATIC, m_bitsPerPixelStatic);
	DDX_Control(pDX, IDC_MAX_SCAN_TIME_STATIC, m_maxScanTimeStatic);
	DDX_Control(pDX, IDC_BITS_PER_CHANNEL_STATIC, m_bitsPerChannelStatic);
	DDX_Control(pDX, IDC_USB_RATE_STATIC, m_usbRateStatic);
}


BEGIN_MESSAGE_MAP(ScanParametersDialog, DialogBarEx)
	ON_BN_CLICKED(IDC_READ_CURRENT_BUTTON, OnBnClickedReadCurrentButton)
	ON_BN_CLICKED(IDC_APPLY_BUTTON, OnBnClickedApplyButton)
	ON_CBN_SELCHANGE(IDC_RESOLUTION_COMBO, OnResolutionComboSelectChanged)
END_MESSAGE_MAP()


// ScanParametersDialog message handlers

BOOL ScanParametersDialog::OnInitDialogBar()
{
	DialogBarEx::OnInitDialogBar();

	// TODO:  Add extra initialization here

	BuildSampleLUT();

	// Initialize controls to default values
	m_modeCombo.SetCurSel(2);
	m_resolutionCombo.SetCurSel(1);
	m_leftOffsetEdit.SetWindowText("0");
	m_topOffsetEdit.SetWindowText("0");
	m_widthEdit.SetWindowText("864");
	m_lengthEdit.SetWindowText("1100");
	m_channelOrderCombo.SetCurSel(1);
	m_brightnessEdit.SetWindowText("0");
	m_contrastEdit.SetWindowText("0");
	m_thresholdEdit.SetWindowText("0");
	m_gammaEdit.SetWindowText("0");
	m_highlightEdit.SetWindowText("0");
	m_shadowEdit.SetWindowText("0");
	m_descreenCheckbox.SetCheck(FALSE);
	m_duplexCheckbox.SetCheck(FALSE);
	m_EOPDetectCheckbox.SetCheck(FALSE);
	m_spoolerCheckbox.SetCheck(TRUE);
	m_planarChunkyCombo.SetCurSel(0);
	m_photoInterpCombo.SetCurSel(0);
	m_feedDirectionCombo.SetCurSel(0);
	m_setScanRateCheckbox.SetCheck(FALSE);
	m_scanRateEdit.SetWindowText("0");
	m_feedRateEdit.SetWindowText("0");
	m_grayLUTCombo.SetCurSel(0);
	m_redLUTCombo.SetCurSel(0);
	m_greenLUTCombo.SetCurSel(0);
	m_blueLUTCombo.SetCurSel(0);
	m_lineWidthInBytesStatic.SetWindowText("Line width in bytes:");
	m_bitsPerPixelStatic.SetWindowText("Bits per pixel:");
	m_bitsPerChannelStatic.SetWindowText("Bits per channel:");
	m_maxScanTimeStatic.SetWindowText("Max scan time:");
	m_usbRateStatic.SetWindowText("USB rate:");

	((CButton*)GetDlgItem(IDC_APPLY_BUTTON))->SetWindowText("Apply");

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void ScanParametersDialog::ShowError(SIProperty &prop)
{

	char *pErrorText;
	g_scanner.SI_GetLastErrorText(&pErrorText);


	CString msg = "Property: ";
	msg += PropertyIDToText(prop.propertyID);
	msg += "\n";
	msg += pErrorText;


	if((g_scanner.SI_GetLastError() == SIR_PROPERTY_INVALID_VALUE)
		&& prop.containerType == SICON_RANGE)
	{
		msg += "\nValid range is ";
	
		char buff[100];
		buff[0] = 0;

		// use sprintf here instead of CString::AppendFormat since it seem to 
		// handle floating point numbers better.
		switch(prop.itemType)
		{
			case SI_INT32:
				sprintf(buff, "%d to %d.", prop.range.minimum.iVal, prop.range.maximum.iVal);
				//msg.AppendFormat("%d to %d.", prop.range.minimum.iVal, prop.range.maximum.iVal);
				break;
			case SI_FLOAT32:
				sprintf(buff, "%.6f to %.6f.", prop.range.minimum.fVal, prop.range.maximum.fVal);
				//msg.AppendFormat("%.6f to %.6f.", prop.range.minimum.fVal, prop.range.maximum.fVal);
		};
		msg += buff;
	}

	MessageBox(msg);

}

void ScanParametersDialog::OnBnClickedReadCurrentButton()
{
	PropertiesToControls();
}


void ScanParametersDialog::OnBnClickedApplyButton()
{
	if(ControlsToProperties())
	{
		PropertiesToControls();
	}
	
	((CButton*)GetDlgItem(IDC_APPLY_BUTTON))->SetWindowText("Apply");

}

void ScanParametersDialog::OnResolutionComboSelectChanged()
{
	// The resolution combo box has just changed. For convenience, we'd
	// like to populate the width edit fields with the maximum values for 
	// the selected resolution. To find out what that value is, we'll go ahead 
	// and change the resolution property in the scanner and then query the width
	// properties. For the length, we don't want the maximum; we just want 
	// it to be 11 inches in whatever resolution was selected.

	SIResult result;
	SIProperty prop;
	CString textbuf;

	// Get the current resolution
	prop.propertyID = SIP_XRESOLUTION;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	int32 oldXRes = prop.list.current.iVal;

	// Set the new X resolution.
	// Note that in the Speedbird scanner, the Y resolution property will change 
	// to match the X resolution property. So we only need to change one.
	//m_resolutionCombo.GetWindowText(textbuf);
	m_resolutionCombo.GetLBText(m_resolutionCombo.GetCurSel(), textbuf);
	int32 newXRes = atoi(textbuf);
	prop.list.current.iVal = newXRes;

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
	}

	// Now query for the maximum scan width
	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	textbuf.Format("%d", prop.range.maximum.iVal);
	m_widthEdit.SetWindowText(textbuf);

	// For the scan length, we don't want the maximum.  Instead we want 11 inches. 
	// So multiply the resolution by 11 and set the length field
	textbuf.Format("%d", newXRes * 11);
	m_lengthEdit.SetWindowText(textbuf);

	// Now restore the previous resolution
	prop.propertyID = SIP_XRESOLUTION;
	prop.containerType = SICON_LIST;
	prop.itemType = SI_INT32;
	prop.list.current.iVal = oldXRes;
	
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
	}

}


void ScanParametersDialog::PropertiesToControls()
{
	SIResult result;
	SIProperty prop;
	int index = 0;
	CString textbuf;

	m_suppressApplyButtonUpdate = true;

	try {

	////////////////// Scan Mode ////////////////////////////////
	prop.propertyID = SIP_SCAN_MODE;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));
		switch(prop.list.current.iVal)
		{
			case SI_SCANMODE_BW:	index = 0; break;
			case SI_SCANMODE_GRAY:	index = 1; break;
			case SI_SCANMODE_RGB:	index = 2; break;
		};
		
		m_modeCombo.SetCurSel(index);
	}
	else
	{
		throw 0;
	}

	////////////////// Resolution ////////////////////////////////
	prop.propertyID = SIP_XRESOLUTION;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));
		switch(prop.list.current.iVal)
		{
			case 75:	index = 0; break;
			case 100:	index = 1; break;
			case 150:	index = 2; break;
			case 200:	index = 3; break;
			case 300:	index = 4; break;
			case 400:	index = 5; break;
			case 500:	index = 6; break;
			case 600:	index = 7; break;
		};
		
		m_resolutionCombo.SetCurSel(index);
	}
	else
	{
		throw 0;
	}

	////////////////// Left Offset ////////////////////////////////
	prop.propertyID = SIP_XOFFSET;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_leftOffsetEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Top Offset ////////////////////////////////
	prop.propertyID = SIP_YOFFSET;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_topOffsetEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}


	////////////////// Scan width ////////////////////////////////
	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_widthEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Scan length ////////////////////////////////
	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_lengthEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Channel order ////////////////////////////////
	prop.propertyID = SIP_CHANNEL_ORDER;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));
		switch(prop.list.current.iVal)
		{
			case SI_CO_RGB:	index = 0; break;
			case SI_CO_BGR:	index = 1; break;
		};
		
		m_channelOrderCombo.SetCurSel(index);
	}
	else
	{
		throw 0;
	}

	////////////////// Brightness ////////////////////////////////
	prop.propertyID = SIP_BRIGHTNESS;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_brightnessEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}


	////////////////// Contrast  ////////////////////////////////
	prop.propertyID = SIP_CONTRAST;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_contrastEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Threshold ////////////////////////////////
	prop.propertyID = SIP_THRESHOLD;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_thresholdEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Gamma ////////////////////////////////
	prop.propertyID = SIP_GAMMA;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_FLOAT32));
		textbuf.Format("%.1f", prop.range.current.fVal);
		m_gammaEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Highlight ////////////////////////////////
	prop.propertyID = SIP_HIGHLIGHT;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_highlightEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Shadow ////////////////////////////////
	prop.propertyID = SIP_SHADOW;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));
		textbuf.Format("%d", prop.range.current.iVal);
		m_shadowEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Descreen ////////////////////////////////
	prop.propertyID = SIP_DESCREEN_ENABLED;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));
		m_descreenCheckbox.SetCheck(prop.single.current.bVal == SI_TRUE);
	}
	else
	{
		throw 0;
	}

	////////////////// Duplex ////////////////////////////////
	prop.propertyID = SIP_DUPLEX_ENABLED;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));
		
		m_duplexCheckbox.EnableWindow();
		m_duplexCheckbox.SetCheck(prop.single.current.bVal == SI_TRUE);
	}
	else if (result == SIR_PROPERTY_UNSUPPORTED)
	{
		// The Duplex property is not supported for single-sided scanners. 
		// This is not an error.
		m_duplexCheckbox.SetCheck(FALSE);
		m_duplexCheckbox.EnableWindow(FALSE);
	}
	else
	{
		throw 0;
	}

	////////////////// End-of-page Detection ////////////////////////////////
	prop.propertyID = SIP_EOP_DETECT_ENABLED;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));
		m_EOPDetectCheckbox.SetCheck(prop.single.current.bVal == SI_TRUE);
	}
	else
	{
		throw 0;
	}

	////////////////// Spooler  ////////////////////////////////
	prop.propertyID = SIP_SPOOLER_ENABLED;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));
		m_spoolerCheckbox.SetCheck(prop.single.current.bVal == SI_TRUE);
	}
	else
	{
		throw 0;
	}

	////////////////// Planar/Chunky ////////////////////////////////
	prop.propertyID = SIP_PLANARCHUNKY;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
		switch(prop.single.current.iVal)
		{	
			// Currently, only chunky format is supported by the driver for the Speedbird
			case SI_PC_CHUNKY:	index = 0; break;
		};
		
		m_planarChunkyCombo.SetCurSel(index);
	}
	else
	{
		throw 0;
	}

	////////////////// Photometric Interpretation ////////////////////////////////
	prop.propertyID = SIP_PHOTOMETRIC_INTERPRETATION;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));
		switch(prop.list.current.iVal)
		{	
			// Currently, only chunky is supported by the driver
			case SI_PI_BLACKZERO:	index = 0; break;
			case SI_PI_WHITEZERO:	index = 1; break;
		};
		
		m_photoInterpCombo.SetCurSel(index);
	}
	else
	{
		throw 0;
	}

	////////////////// Feed Direction  ////////////////////////////////
	prop.propertyID = SIP_FEED_DIRECTION;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));
		switch(prop.list.current.iVal)
		{	
			// Currently, only chunky is supported by the driver
			case SI_FORWARD:	index = 0; break;
			case SI_REVERSE:	index = 1; break;
		};
		
		m_feedDirectionCombo.SetCurSel(index);
	}
	else
	{
		throw 0;
	}

	////////////////// Scan Rate  ////////////////////////////////
	m_setScanRateCheckbox.SetCheck(FALSE);

	prop.propertyID = SIP_SCAN_RATE;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_FLOAT32));
		textbuf.Format("%.5f", prop.range.current.fVal);
		m_scanRateEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Feed Rate  ////////////////////////////////

	prop.propertyID = SIP_FEED_RATE;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_FLOAT32));
		textbuf.Format("%.5f", prop.range.current.fVal);
		m_feedRateEdit.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}


	////////////////// Grayscale LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_GRAY;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));
		m_grayLUTCombo.SetCurSel((prop.array.numItems == 0) ? 0 : 1);
	}
	else
	{
		throw 0;
	}

	////////////////// Red LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_RED;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));
		m_redLUTCombo.SetCurSel((prop.array.numItems == 0) ? 0 : 1);
	}
	else
	{
		throw 0;
	}

	////////////////// Green LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_GREEN;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));
		m_greenLUTCombo.SetCurSel((prop.array.numItems == 0) ? 0 : 1);
	}
	else
	{
		throw 0;
	}

	////////////////// Blue LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_BLUE;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));
		m_blueLUTCombo.SetCurSel((prop.array.numItems == 0) ? 0 : 1);
	}
	else
	{
		throw 0;
	}

	////////////////// Line width in bytes ////////////////////////////////
	//
	// This is a read-only property.
	//
	prop.propertyID = SIP_LINE_WIDTH_IN_BYTES;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
		textbuf.Format("Line width in bytes: %d", prop.single.current.iVal);
		m_lineWidthInBytesStatic.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Bits per pixel ////////////////////////////////
	//
	// Bits per pixel is a read-only value because there is only one possible 
	// value for each scan mode in the Speedbird scanner.
	// For color, BBP = 24.
	// For grayscale, BBP = 8.
	// For B&W, BBP = 1.
	//
	prop.propertyID = SIP_BITS_PER_PIXEL;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
		textbuf.Format("Bits per pixel: %d", prop.single.current.iVal);
		m_bitsPerPixelStatic.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Bits per channel ////////////////////////////////
	//
	// Bits per channel is a read-only value because there is only one possible 
	// value for each scan mode in the Speedbird.
	// For color, BBP = 8.
	// For grayscale, BBP = 8.
	// For B&W, BBP = 1.
	//
	prop.propertyID = SIP_BITS_PER_CHANNEL;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
		textbuf.Format("Bits per channel: %d", prop.single.current.iVal);
		m_bitsPerChannelStatic.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}

	////////////////// Maximum scan time in seconds ////////////////////////////////
	prop.propertyID = SIP_MAX_SCAN_TIME_IN_SEC;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
		textbuf.Format("Max scan time: %d seconds", prop.single.current.iVal);
		m_maxScanTimeStatic.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}


	////////////////// USB rate ////////////////////////////////
	prop.propertyID = SIP_USB_RATE;
	result = g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_INT32));
		textbuf = "USB rate:  ";
		if(prop.single.current.iVal == SI_USB_HIGH)
		{
			textbuf += "HIGH";
		}
		else if(prop.single.current.iVal == SI_USB_FULL)
		{
			textbuf += "FULL";
		}

		m_usbRateStatic.SetWindowText(textbuf);
	}
	else
	{
		throw 0;
	}


	} 
	catch(...)
	{
		ShowError(prop);
		m_suppressApplyButtonUpdate = false;
		return;
	}

	m_suppressApplyButtonUpdate = false;

	((CButton*)GetDlgItem(IDC_APPLY_BUTTON))->SetWindowText("Apply");

}


bool ScanParametersDialog::ControlsToProperties()
{
	SIResult result;
	SIProperty prop;
	int index = 0;
	CString textbuf;

	////////////////// Scan Mode ////////////////////////////////
	prop.propertyID = SIP_SCAN_MODE;
	prop.containerType = SICON_LIST;
	prop.itemType = SI_INT32;

	switch(m_modeCombo.GetCurSel())
	{
		case 0: prop.list.current.iVal = SI_SCANMODE_BW; break;
		case 1: prop.list.current.iVal = SI_SCANMODE_GRAY; break;
		case 2: prop.list.current.iVal = SI_SCANMODE_RGB; break;
	}

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Resolution ////////////////////////////////
	//
	//	Only the XResolution property is set. The YResolution property 
	//	will follow the XResolution property for the Speedbird scanner.
	//	We can check that below.
	prop.propertyID = SIP_XRESOLUTION;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	m_resolutionCombo.GetWindowText(textbuf);
	prop.list.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	// Check YResolution to ensure that it matches the XResolution.
	int32 xRes = prop.list.current.iVal;			// save x resolution value 

	prop.propertyID = SIP_YRESOLUTION;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));
	
	if(prop.list.current.iVal != xRes)		// compare x resolution to y resolution
	{
		// This would be an error in the driver DLL.
		MessageBox("Y resolution does not match X resolution.");
	}

	////////////////// Left Offset ////////////////////////////////
	prop.propertyID = SIP_XOFFSET;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_leftOffsetEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Top Offset ////////////////////////////////
	prop.propertyID = SIP_YOFFSET;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_topOffsetEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Scan width ////////////////////////////////
	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_widthEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}


	////////////////// Scan length ////////////////////////////////
	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_lengthEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}


	////////////////// Channel order ////////////////////////////////
	prop.propertyID = SIP_CHANNEL_ORDER;
	g_scanner.SI_GetProperty(&prop);	// calling GetProperty sets up the correct container and item type.
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	switch(m_channelOrderCombo.GetCurSel())
	{
		case 0: prop.list.current.iVal = SI_CO_RGB; break;
		case 1: prop.list.current.iVal = SI_CO_BGR; break;
	}

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Brightness ////////////////////////////////
	prop.propertyID = SIP_BRIGHTNESS;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_brightnessEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Contrast  ////////////////////////////////
	prop.propertyID = SIP_CONTRAST;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_contrastEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Threshold ////////////////////////////////
	prop.propertyID = SIP_THRESHOLD;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_thresholdEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Gamma ////////////////////////////////
	prop.propertyID = SIP_GAMMA;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_FLOAT32));

	m_gammaEdit.GetWindowText(textbuf);
	prop.range.current.fVal = (float32)atof(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}


	////////////////// Highlight ////////////////////////////////
	prop.propertyID = SIP_HIGHLIGHT;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_highlightEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Shadow ////////////////////////////////
	prop.propertyID = SIP_SHADOW;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_INT32));

	m_shadowEdit.GetWindowText(textbuf);
	prop.range.current.iVal = atoi(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Descreen ////////////////////////////////
	prop.propertyID = SIP_DESCREEN_ENABLED;
	g_scanner.SI_GetProperty(&prop);
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));

		// Get descreen state from the checkbox
		prop.single.current.bVal = m_descreenCheckbox.GetCheck() ? SI_TRUE : SI_FALSE;

		// Set descreen state to the scanner
		result = g_scanner.SI_SetProperty(&prop);
		if(result != SIR_SUCCESS)
		{
			ShowError(prop);
			return false;
		}
	}

	////////////////// Duplex ////////////////////////////////
	prop.propertyID = SIP_DUPLEX_ENABLED;
	result = g_scanner.SI_GetProperty(&prop);
	
	// The Duplex property is not supported for single-sided scanners. 
	// This is not an error.
	if(result == SIR_SUCCESS)
	{
		assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));

		// Get duplex state from the checkbox
		prop.single.current.bVal = m_duplexCheckbox.GetCheck() ? SI_TRUE : SI_FALSE;

		// Set duplex state to the scanner
		result = g_scanner.SI_SetProperty(&prop);
		if(result != SIR_SUCCESS)
		{
			ShowError(prop);
			return false;
		}
	}

	////////////////// End-of-page Detection ////////////////////////////////
	prop.propertyID = SIP_EOP_DETECT_ENABLED;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));

	prop.single.current.bVal = m_EOPDetectCheckbox.GetCheck() ? SI_TRUE : SI_FALSE;
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Spooler  ////////////////////////////////
	prop.propertyID = SIP_SPOOLER_ENABLED;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_SINGLE) && (prop.itemType == SI_BOOL));

	prop.single.current.bVal = m_spoolerCheckbox.GetCheck() ? SI_TRUE : SI_FALSE;
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Planar/Chunky ////////////////////////////////
	//
	//	The SIP_PLANARCHUNKY property is not set here since only one 
	//	possible value is supported for the Speedbird.
	/////////////////////////////////////////////////////////////////


	////////////////// Photometric Interpretation ////////////////////////////////
	prop.propertyID = SIP_PHOTOMETRIC_INTERPRETATION;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	switch(m_photoInterpCombo.GetCurSel())
	{
		case 0: prop.list.current.iVal = SI_PI_BLACKZERO; break;
		case 1: prop.list.current.iVal = SI_PI_WHITEZERO; break;
	};
	
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}


	////////////////// Feed Direction  ////////////////////////////////
	prop.propertyID = SIP_FEED_DIRECTION;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	switch(m_feedDirectionCombo.GetCurSel())
	{
		case 0: prop.list.current.iVal = SI_FORWARD; break;
		case 1: prop.list.current.iVal = SI_REVERSE; break;
	};
	
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Scan Rate ////////////////////////////////

	if(m_setScanRateCheckbox.GetCheck())
	{

		m_setScanRateCheckbox.SetCheck(FALSE);

		prop.propertyID = SIP_SCAN_RATE;
		g_scanner.SI_GetProperty(&prop);
		assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_FLOAT32));

		m_scanRateEdit.GetWindowText(textbuf);
		prop.range.current.fVal = (float32)atof(textbuf);

		result = g_scanner.SI_SetProperty(&prop);
		if(result != SIR_SUCCESS)
		{
			ShowError(prop);
			return false;
		}
	}

	////////////////// Feed Rate ////////////////////////////////

	prop.propertyID = SIP_FEED_RATE;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_RANGE) && (prop.itemType == SI_FLOAT32));

	m_feedRateEdit.GetWindowText(textbuf);
	prop.range.current.fVal = (float32)atof(textbuf);

	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Grayscale LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_GRAY;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));

	switch(m_grayLUTCombo.GetCurSel())
	{
		case 0:		prop.array.numItems = 0;
					prop.array.items.pusVal = NULL;
					break;
		case 1:		prop.array.numItems = 256;
					prop.array.items.pusVal = m_lut;
					break;
	};
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Red LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_RED;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));

	switch(m_redLUTCombo.GetCurSel())
	{
		case 0:		prop.array.numItems = 0;
					prop.array.items.pusVal = NULL;
					break;
		case 1:		prop.array.numItems = 256;
					prop.array.items.pusVal = m_lut;
					break;
	};
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}


	////////////////// Green LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_GREEN;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));

	switch(m_greenLUTCombo.GetCurSel())
	{
		case 0:		prop.array.numItems = 0;
					prop.array.items.pusVal = NULL;
					break;
		case 1:		prop.array.numItems = 256;
					prop.array.items.pusVal = m_lut;
					break;
	};
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	////////////////// Blue LUT  ////////////////////////////////
	prop.propertyID = SIP_LUT_BLUE;
	g_scanner.SI_GetProperty(&prop);
	assert((prop.containerType == SICON_ARRAY) && (prop.itemType == SI_UINT16));

	switch(m_blueLUTCombo.GetCurSel())
	{
		case 0:		prop.array.numItems = 0;
					prop.array.items.pusVal = NULL;
					break;
		case 1:		prop.array.numItems = 256;
					prop.array.items.pusVal = m_lut;
					break;
	};
	result = g_scanner.SI_SetProperty(&prop);
	if(result != SIR_SUCCESS)
	{
		ShowError(prop);
		return false;
	}

	return true;

}


void ScanParametersDialog::BuildSampleLUT()
{
	for(int i=0; i<256; i++)
	{
		m_lut[i] = (i << 8)/2;
	}
}

char *ScanParametersDialog::PropertyIDToText(int propertyID)
{

	switch(propertyID)
	{
		case SIP_OPTICAL_RESOLUTION:		return "SIP_OPTICAL_RESOLUTION";
		case SIP_OPTICAL_WIDTH_IN_PIXELS:	return "SIP_OPTICAL_WIDTH_IN_PIXELS";
		case SIP_XRESOLUTION:				return "SIP_XRESOLUTION";
		case SIP_YRESOLUTION:				return "SIP_YRESOLUTION";
		case SIP_BITS_PER_CHANNEL:			return "SIP_BITS_PER_CHANNEL";
		case SIP_BITS_PER_PIXEL:			return "SIP_BITS_PER_PIXEL";
		case SIP_SCAN_MODE:					return "SIP_SCAN_MODE";
		case SIP_BRIGHTNESS:				return "SIP_BRIGHTNESS";
		case SIP_CONTRAST:					return "SIP_CONTRAST";
		case SIP_PHOTOMETRIC_INTERPRETATION:	return "SIP_PHOTOMETRIC_INTERPRETATION";
		case SIP_YOFFSET:					return "SIP_YOFFSET";
		case SIP_XOFFSET:					return "SIP_XOFFSET";
		case SIP_SCAN_WIDTH_IN_PIXELS:		return "SIP_SCAN_WIDTH_IN_PIXELS";
		case SIP_SCAN_LENGTH_IN_LINES:		return "SIP_SCAN_LENGTH_IN_LINES";
		case SIP_PLANARCHUNKY:				return "SIP_PLANARCHUNKY";
		case SIP_CHANNEL_ORDER:				return "SIP_CHANNEL_ORDER";
		case SIP_MAX_SCAN_TIME_IN_SEC:		return "SIP_MAX_SCAN_TIME_IN_SEC";
		case SIP_LINE_WIDTH_IN_BYTES:		return "SIP_LINE_WIDTH_IN_BYTES";
		case SIP_DESCREEN_ENABLED:			return "SIP_DESCREEN_ENABLED";
		case SIP_DUPLEX_ENABLED:			return "SIP_DUPLEX_ENABLED";
		case SIP_SPOOLER_ENABLED:			return "SIP_SPOOLER_ENABLED";
		case SIP_FEED_DIRECTION:			return "SIP_FEED_DIRECTION";
		case SIP_OPTICAL_LENGTH_IN_PIXELS:	return "SIP_OPTICAL_LENGTH_IN_PIXELS";
		case SIP_GAMMA:						return "SIP_GAMMA";
		case SIP_HIGHLIGHT:					return "SIP_HIGHLIGHT";
		case SIP_SHADOW:					return "SIP_SHADOW";
		case SIP_LUT_GRAY:					return "SIP_LUT_GRAY";
		case SIP_LUT_RED:					return "SIP_LUT_RED";
		case SIP_LUT_GREEN:					return "SIP_LUT_GREEN";
		case SIP_LUT_BLUE:					return "SIP_LUT_BLUE";
		case SIP_THRESHOLD:					return "SIP_THRESHOLD";
		case SIP_EOP_DETECT_ENABLED:		return "SIP_EOP_DETECT_ENABLED";
		case SIP_SCAN_RATE:					return "SIP_SCAN_RATE";
		case SIP_FEED_RATE:					return "SIP_FEED_RATE";
		case SIP_USB_RATE:					return "SIP_USB_RATE";
	}
	
	return "Invalid property ID";
}



BOOL ScanParametersDialog::OnCommand(WPARAM wParam, LPARAM lParam)
{
	// TODO: Add your specialized code here and/or call the base class

	if(!m_suppressApplyButtonUpdate)
	{
		// This handles the edit boxes and combo boxes.
		if((HIWORD(wParam) == EN_CHANGE) || (HIWORD(wParam) == CBN_SELCHANGE))
		{
			((CButton*)GetDlgItem(IDC_APPLY_BUTTON))->SetWindowText("*Apply*");
		}

		// The check boxes work the same as buttons. But we don't want to 
		// react to the [Apply] button itself or the [Read Current] button.
		// So react to any button click other than those two.
		if((HIWORD(wParam) == BN_CLICKED) && (LOWORD(wParam) != IDC_APPLY_BUTTON) 
			&& (LOWORD(wParam) != IDC_READ_CURRENT_BUTTON))
		{
			((CButton*)GetDlgItem(IDC_APPLY_BUTTON))->SetWindowText("*Apply*");
		}
	}

	return DialogBarEx::OnCommand(wParam, lParam);
}
